@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_all_models.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_all_models.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -95,6 +96,7 @@ from camcops_server.cc_modules.cc_user import (
     User,
 )
 
+
 # =============================================================================
 # Task imports
 # =============================================================================
@@ -110,6 +112,7 @@ from camcops_server.cc_modules.cc_user import (
 # noinspection PyUnresolvedReferences
 from camcops_server.tasks import *  # see tasks/__init__.py  # noqa: F401,F403
 
+
 # =============================================================================
 # Other report imports
 # =============================================================================
@@ -117,13 +120,15 @@ from camcops_server.tasks import *  # see tasks/__init__.py  # noqa: F401,F403
 # noinspection PyUnresolvedReferences
 from camcops_server.cc_modules.cc_taskreports import TaskCountReport  # noqa: E501,F401
 
+
 # =============================================================================
 # Logging
 # =============================================================================
 
 log = BraceStyleAdapter(logging.getLogger(__name__))
 
-# log.critical("Loading cc_all_models")
+# log.debug("Loading cc_all_models")
+
 
 # =============================================================================
 # Ensure that anything with an AbstractConcreteBase gets its mappers
@@ -132,6 +137,7 @@ log = BraceStyleAdapter(logging.getLogger(__name__))
 
 configure_mappers()
 
+
 # =============================================================================
 # Tables (and fields) that clients can't touch
 # =============================================================================
@@ -163,7 +169,7 @@ RESERVED_TABLE_NAMES = [
     User.__tablename__,
     UserGroupMembership.__tablename__,
 ]
-RESERVED_FIELDS = GenericTabletRecordMixin.RESERVED_FIELDS
+
 
 # =============================================================================
 # Catalogue tables that clients use
