@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_client_api_core.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_client_api_core.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -450,7 +451,7 @@ class UploadRecordResult(object):
         """
         Returns all PKs (old, new, or both).
         """
-        return list(x for x in [self.oldserverpk, self.newserverpk]
+        return list(x for x in (self.oldserverpk, self.newserverpk)
                     if x is not None)
 
     @property
