@@ -1,11 +1,13 @@
 #!/usr/bin/env python
 
+# noinspection HttpUrlsUsage
 """
 camcops_server/cc_modules/cc_config.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +22,7 @@ camcops_server/cc_modules/cc_config.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -75,8 +77,10 @@ import datetime
 import os
 import logging
 import re
+from subprocess import run, PIPE
 from typing import Any, Dict, Generator, List, Optional, Union
 
+from cardinal_pythonlib.classes import class_attribute_values
 from cardinal_pythonlib.configfiles import (
     get_config_parameter,
     get_config_parameter_boolean,
@@ -118,12 +122,15 @@ from camcops_server.cc_modules.cc_constants import (
     CONFIG_FILE_EXPORT_SECTION,
     CONFIG_FILE_SERVER_SECTION,
     CONFIG_FILE_SITE_SECTION,
+    CONFIG_FILE_SMS_BACKEND_PREFIX,
     ConfigDefaults,
     ConfigParamExportGeneral,
     ConfigParamExportRecipient,
     ConfigParamServer,
     ConfigParamSite,
     DockerConstants,
+    MfaMethod,
+    SmsBackendNames,
 )
 from camcops_server.cc_modules.cc_exportrecipientinfo import (
     ExportRecipientInfo,
@@ -134,6 +141,11 @@ from camcops_server.cc_modules.cc_filename import (
 )
 from camcops_server.cc_modules.cc_language import POSSIBLE_LOCALES
 from camcops_server.cc_modules.cc_pyramid import MASTER_ROUTE_CLIENT_API
+from camcops_server.cc_modules.cc_sms import (
+    get_sms_backend,
+    KapowSmsBackend,
+    TwilioSmsBackend,
+)
 from camcops_server.cc_modules.cc_snomed import (
     get_all_task_snomed_concepts,
     get_icd9_snomed_concepts_from_xml,
@@ -221,11 +233,9 @@ def warn_if_not_within_docker_dir(param_name: str,
         for d in permitted_dirs
     )
     if not ok:
-        log.warning(
-            f"Config parameter {param_name} {is_phrase} {filespec!r}, "
-            f"which is not within the permitted Docker directories "
-            f"{permitted_dirs!r}"
-        )
+        log.warning(f"Config parameter {param_name} {is_phrase} {filespec!r}, "
+                    f"which is not within the permitted Docker directories "
+                    f"{permitted_dirs!r}")
 
 
 def warn_if_not_docker_value(param_name: str,
@@ -244,10 +254,9 @@ def warn_if_not_docker_value(param_name: str,
             Value that should be used.
     """
     if actual_value != required_value:
-        log.warning(
-            f"Config parameter {param_name} is {actual_value!r}, "
-            f"but should be {required_value!r} when running inside Docker"
-        )
+        log.warning(f"Config parameter {param_name} is {actual_value!r}, "
+                    f"but should be {required_value!r} when running inside "
+                    f"Docker")
 
 
 def warn_if_not_present(param_name: str, value: Any) -> None:
@@ -262,10 +271,21 @@ def warn_if_not_present(param_name: str, value: Any) -> None:
             Value in the config file.
     """
     if value is None or value == "":
-        log.warning(
-            f"Config parameter {param_name} is not specified, "
-            f"but should be specified when running inside Docker"
-        )
+        log.warning(f"Config parameter {param_name} is not specified, "
+                    f"but should be specified when running inside Docker")
+
+
+def list_to_multiline_string(values: List[Any]) -> str:
+    """
+    Converts a Python list to a multiline string suitable for use as a config
+    file default (in a pretty way).
+    """
+    spacer = "\n    "
+    gen_values = (str(x) for x in values)
+    if len(values) <= 1:
+        return spacer.join(gen_values)
+    else:
+        return spacer + spacer.join(gen_values)
 
 
 # =============================================================================
@@ -275,7 +295,7 @@ def warn_if_not_present(param_name: str, value: Any) -> None:
 # Cosmetic demonstration constants:
 DEFAULT_DB_READONLY_USER = 'QQQ_USERNAME_REPLACE_ME'
 DEFAULT_DB_READONLY_PASSWORD = 'PPP_PASSWORD_REPLACE_ME'
-DUMMY_INSTITUTION_URL = 'http://www.mydomain/'
+DUMMY_INSTITUTION_URL = 'https://www.mydomain/'
 
 
 def get_demo_config(for_docker: bool = False) -> str:
@@ -331,12 +351,21 @@ def get_demo_config(for_docker: bool = False) -> str:
 
 {ConfigParamSite.WKHTMLTOPDF_FILENAME} =
 
+# -----------------------------------------------------------------------------
+# Server geographical location
+# -----------------------------------------------------------------------------
+
+{ConfigParamSite.REGION_CODE} = {cd.REGION_CODE}
+
 # -----------------------------------------------------------------------------
 # Login and session configuration
 # -----------------------------------------------------------------------------
 
+{ConfigParamSite.MFA_METHODS} = {list_to_multiline_string(cd.MFA_METHODS)}
+{ConfigParamSite.MFA_TIMEOUT_S} = {cd.MFA_TIMEOUT_S}
 {ConfigParamSite.SESSION_COOKIE_SECRET} = camcops_autogenerated_secret_{session_cookie_secret}
 {ConfigParamSite.SESSION_TIMEOUT_MINUTES} = {cd.SESSION_TIMEOUT_MINUTES}
+{ConfigParamSite.SESSION_CHECK_USER_IP} = {cd.SESSION_CHECK_USER_IP}
 {ConfigParamSite.PASSWORD_CHANGE_FREQUENCY_DAYS} = {cd.PASSWORD_CHANGE_FREQUENCY_DAYS}
 {ConfigParamSite.LOCKOUT_THRESHOLD} = {cd.LOCKOUT_THRESHOLD}
 {ConfigParamSite.LOCKOUT_DURATION_INCREMENT_MINUTES} = {cd.LOCKOUT_DURATION_INCREMENT_MINUTES}
@@ -366,6 +395,12 @@ def get_demo_config(for_docker: bool = False) -> str:
 {ConfigParamSite.EMAIL_SENDER} =
 {ConfigParamSite.EMAIL_REPLY_TO} = CamCOPS clinical administrator <admin@myinstitution.mydomain>
 
+# -----------------------------------------------------------------------------
+# SMS options
+# -----------------------------------------------------------------------------
+
+{ConfigParamSite.SMS_BACKEND} = {cd.SMS_BACKEND}
+
 # -----------------------------------------------------------------------------
 # User download options
 # -----------------------------------------------------------------------------
@@ -430,6 +465,15 @@ def get_demo_config(for_docker: bool = False) -> str:
     HTTP_X_FORWARDED_FOR
     HTTP_X_SCRIPT_NAME
 
+# -----------------------------------------------------------------------------
+# Determining the externally accessible CamCOPS URL for back-end work
+# -----------------------------------------------------------------------------
+
+{ConfigParamServer.EXTERNAL_URL_SCHEME} =
+{ConfigParamServer.EXTERNAL_SERVER_NAME} =
+{ConfigParamServer.EXTERNAL_SERVER_PORT} =
+{ConfigParamServer.EXTERNAL_SCRIPT_NAME} =
+
 # -----------------------------------------------------------------------------
 # CherryPy options
 # -----------------------------------------------------------------------------
@@ -552,7 +596,17 @@ def get_demo_config(for_docker: bool = False) -> str:
 {ConfigParamExportRecipient.EMAIL_KEEP_MESSAGE} = {cd.HL7_KEEP_MESSAGE}
 
     # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-    # Options applicable to HL7
+    # Options applicable to FHIR
+    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+{ConfigParamExportRecipient.FHIR_API_URL} = https://my.fhir.server/api
+{ConfigParamExportRecipient.FHIR_APP_ID} = {cd.FHIR_APP_ID}
+{ConfigParamExportRecipient.FHIR_APP_SECRET} = my_fhir_secret_abc
+{ConfigParamExportRecipient.FHIR_LAUNCH_TOKEN} =
+{ConfigParamExportRecipient.FHIR_CONCURRENT} =
+
+    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+    # Options applicable to HL7 (v2) exports
     # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
 {ConfigParamExportRecipient.HL7_HOST} = myhl7server.mydomain
@@ -592,6 +646,21 @@ def get_demo_config(for_docker: bool = False) -> str:
 {ConfigParamExportRecipient.REDCAP_API_KEY} = myapikey
 {ConfigParamExportRecipient.REDCAP_FIELDMAP_FILENAME} = /location/of/fieldmap.xml
 
+# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+# Example SMS Backends. No configuration needed for '{SmsBackendNames.CONSOLE}' (testing only).
+# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+[{CONFIG_FILE_SMS_BACKEND_PREFIX}:{SmsBackendNames.KAPOW}]
+
+{KapowSmsBackend.PARAM_USERNAME} = myusername
+{KapowSmsBackend.PARAM_PASSWORD} = mypassword
+
+[{CONFIG_FILE_SMS_BACKEND_PREFIX}:{SmsBackendNames.TWILIO}]
+
+{TwilioSmsBackend.PARAM_SID.upper()} = mysid
+{TwilioSmsBackend.PARAM_TOKEN.upper()} = mytoken
+{TwilioSmsBackend.PARAM_FROM_PHONE_NUMBER.upper()} = myphonenumber
+
     """.strip()  # noqa
 
 
@@ -681,6 +750,7 @@ def get_demo_apache_config(
     cd = ConfigDefaults()
     specimen_internal_port = specimen_internal_port or cd.PORT
     urlbase = "/" + rootpath
+    # noinspection HttpUrlsUsage
     return f"""
 # Demonstration Apache config file section for CamCOPS.
 # Created by CamCOPS version {CAMCOPS_SERVER_VERSION_STRING}.
@@ -1078,9 +1148,14 @@ class CamcopsConfig(object):
                 special circumstances); overrides ``config_filename``
         """
         def _get_str(section: str, paramname: str,
-                     default: str = None) -> Optional[str]:
-            return get_config_parameter(
+                     default: str = None,
+                     replace_empty_strings_with_default: bool = True) \
+                -> Optional[str]:
+            p = get_config_parameter(
                 parser, section, paramname, str, default)
+            if p == "" and replace_empty_strings_with_default:
+                return default
+            return p
 
         def _get_bool(section: str, paramname: str, default: bool) -> bool:
             return get_config_parameter_boolean(
@@ -1123,7 +1198,7 @@ class CamcopsConfig(object):
                 raise AssertionError(
                     f"Environment variable {ENVVAR_CONFIG_FILE} not specified "
                     f"(and no command-line alternative given)")
-            log.info("Reading from config file: {!r}", config_filename)
+            log.info("Reading from config file: {}", config_filename)
             with codecs.open(config_filename, "r", "utf8") as file:
                 parser.read_file(file)
 
@@ -1160,6 +1235,13 @@ class CamcopsConfig(object):
         self.email_host_username = _get_str(s, cs.EMAIL_HOST_USERNAME, "")
         self.email_host_password = _get_str(s, cs.EMAIL_HOST_PASSWORD, "")
 
+        # Development only: Read password from safe using GNU Pass
+        gnu_pass_lookup = _get_str(s, cs.EMAIL_HOST_PASSWORD_GNU_PASS_LOOKUP,
+                                   "")
+        if gnu_pass_lookup:
+            output = run(["pass", gnu_pass_lookup], stdout=PIPE)
+            self.email_host_password = output.stdout.decode("utf-8").split()[0]
+
         self.email_from = _get_str(s, cs.EMAIL_FROM, "")
         self.email_sender = _get_str(s, cs.EMAIL_SENDER, "")
         self.email_reply_to = _get_str(s, cs.EMAIL_REPLY_TO, "")
@@ -1181,6 +1263,19 @@ class CamcopsConfig(object):
             s, cs.LOCKOUT_DURATION_INCREMENT_MINUTES,
             cd.LOCKOUT_DURATION_INCREMENT_MINUTES)
 
+        self.mfa_methods = _get_multiline(s, cs.MFA_METHODS)
+        if not self.mfa_methods:
+            self.mfa_methods = cd.MFA_METHODS
+            log.warning(f"MFA_METHODS not specified. Using {self.mfa_methods}")
+        self.mfa_methods = [x.lower() for x in self.mfa_methods]
+        assert self.mfa_methods, "Bug: missing MFA_METHODS"
+        _valid_mfa_methods = class_attribute_values(MfaMethod)
+        for _mfa_method in self.mfa_methods:
+            if _mfa_method not in _valid_mfa_methods:
+                raise ValueError(f"Bad MFA_METHOD item: {_mfa_method!r}")
+
+        self.mfa_timeout_s = _get_int(s, cs.MFA_TIMEOUT_S, cd.MFA_TIMEOUT_S)
+
         self.password_change_frequency_days = _get_int(
             s, cs.PASSWORD_CHANGE_FREQUENCY_DAYS,
             cd.PASSWORD_CHANGE_FREQUENCY_DAYS)
@@ -1193,6 +1288,7 @@ class CamcopsConfig(object):
         # currently not configurable, but easy to add in the future:
         self.plot_fontsize = cd.PLOT_FONTSIZE
 
+        self.region_code = _get_str(s, cs.REGION_CODE, cd.REGION_CODE)
         self.restricted_tasks = {}  # type: Dict[str, List[str]]
         # ... maps XML task names to lists of authorized group names
         restricted_tasks = _get_multiline(s, cs.RESTRICTED_TASKS)
@@ -1223,6 +1319,11 @@ class CamcopsConfig(object):
         self.session_cookie_secret = _get_str(s, cs.SESSION_COOKIE_SECRET)
         self.session_timeout = datetime.timedelta(
             minutes=self.session_timeout_minutes)
+        self.session_check_user_ip = _get_bool(s, cs.SESSION_CHECK_USER_IP,
+                                               cd.SESSION_CHECK_USER_IP)
+        sms_label = _get_str(s, cs.SMS_BACKEND, cd.SMS_BACKEND)
+        sms_config = self._read_sms_config(parser, sms_label)
+        self.sms_backend = get_sms_backend(sms_label, sms_config)
         self.snomed_task_xml_filename = _get_str(
             s, cs.SNOMED_TASK_XML_FILENAME)
         self.snomed_icd9_xml_filename = _get_str(
@@ -1315,6 +1416,16 @@ class CamcopsConfig(object):
             ws, cw.TRUSTED_PROXY_HEADERS)
         self.unix_domain_socket = _get_str(ws, cw.UNIX_DOMAIN_SOCKET)
 
+        # The defaults here depend on values above:
+        self.external_url_scheme = _get_str(
+            ws, cw.EXTERNAL_URL_SCHEME, cd.EXTERNAL_URL_SCHEME)
+        self.external_server_name = _get_str(
+            ws, cw.EXTERNAL_SERVER_NAME, self.host)
+        self.external_server_port = _get_int(
+            ws, cw.EXTERNAL_SERVER_PORT, self.port)
+        self.external_script_name = _get_str(
+            ws, cw.EXTERNAL_SCRIPT_NAME, "")
+
         for tph in self.trusted_proxy_headers:
             if tph not in ReverseProxiedMiddleware.ALL_CANDIDATES:
                 raise ValueError(
@@ -1696,7 +1807,7 @@ class CamcopsConfig(object):
     # File-based locks
     # -------------------------------------------------------------------------
 
-    def get_export_lockfilename_db(self, recipient_name: str) -> str:
+    def get_export_lockfilename_recipient_db(self, recipient_name: str) -> str:
         """
         Returns a full path to a lockfile suitable for locking for a
         whole-database export to a particular export recipient.
@@ -1711,8 +1822,28 @@ class CamcopsConfig(object):
         # ".lock" is appended automatically by the lockfile package
         return os.path.join(self.export_lockdir, filename)
 
-    def get_export_lockfilename_task(self, recipient_name: str,
-                                     basetable: str, pk: int) -> str:
+    def get_export_lockfilename_recipient_fhir(self,
+                                               recipient_name: str) -> str:
+        """
+        Returns a full path to a lockfile suitable for locking for a
+        FHIR export to a particular export recipient.
+
+        (This must be different from
+        :meth:`get_export_lockfilename_recipient_db`, because of what we assume
+        about someone else holding the same lock.)
+
+        Args:
+            recipient_name: name of the recipient
+
+        Returns:
+            a filename
+        """
+        filename = f"camcops_export_fhir_{recipient_name}"
+        # ".lock" is appended automatically by the lockfile package
+        return os.path.join(self.export_lockdir, filename)
+
+    def get_export_lockfilename_recipient_task(self, recipient_name: str,
+                                               basetable: str, pk: int) -> str:
         """
         Returns a full path to a lockfile suitable for locking for a
         single-task export to a particular export recipient.
@@ -1751,6 +1882,27 @@ class CamcopsConfig(object):
         filename = "camcops_celerybeat.pid"
         return os.path.join(self.export_lockdir, filename)
 
+    # -------------------------------------------------------------------------
+    # SMS backend
+    # -------------------------------------------------------------------------
+
+    @staticmethod
+    def _read_sms_config(
+            parser: configparser.ConfigParser,
+            sms_label: str) -> Dict[str, str]:
+        """
+        Read a config section for a specific SMS backend.
+        """
+        section_name = f"{CONFIG_FILE_SMS_BACKEND_PREFIX}:{sms_label}"
+        if not parser.has_section(section_name):
+            return {}
+
+        sms_config = {}
+        section = parser[section_name]
+        for key in section:
+            sms_config[key.lower()] = section[key]
+        return sms_config
+
 
 # =============================================================================
 # Get config filename from an appropriate environment (WSGI or OS)
