@@ -1,11 +1,13 @@
 #!/usr/bin/env python
 
+# noinspection HttpUrlsUsage
 """
 camcops_server/cc_modules/cc_export.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,18 +22,18 @@ camcops_server/cc_modules/cc_export.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
-.. _ActiveMQ: http://activemq.apache.org/
+.. _ActiveMQ: https://activemq.apache.org/
 .. _AMQP: https://www.amqp.org/
 .. _APScheduler: https://apscheduler.readthedocs.io/
-.. _Celery: http://www.celeryproject.org/
+.. _Celery: https://www.celeryproject.org/
 .. _Dramatiq: https://dramatiq.io/
 .. _RabbitMQ: https://www.rabbitmq.com/
 .. _Redis: https://redis.io/
-.. _ZeroMQ: http://zeromq.org/
+.. _ZeroMQ: https://zeromq.org/
 
 **Export and research dump functions.**
 
@@ -149,6 +151,8 @@ Thoughts as of 2018-12-22.
 
 """  # noqa
 
+from contextlib import ExitStack
+import json
 import logging
 import os
 import sqlite3
@@ -189,9 +193,10 @@ from sqlalchemy.sql.schema import Column, MetaData, Table
 from sqlalchemy.sql.sqltypes import Text
 
 from camcops_server.cc_modules.cc_audit import audit
-from camcops_server.cc_modules.cc_constants import DateFormat
+from camcops_server.cc_modules.cc_constants import DateFormat, JSON_INDENT
 from camcops_server.cc_modules.cc_dump import copy_tasks_and_summaries
 from camcops_server.cc_modules.cc_email import Email
+from camcops_server.cc_modules.cc_exception import FhirExportException
 from camcops_server.cc_modules.cc_exportmodels import (
     ExportedTask,
     ExportRecipient,
@@ -208,6 +213,7 @@ from camcops_server.cc_modules.celery import (
     create_user_download,
     email_basic_dump,
     export_task_backend,
+    jittered_delay_s,
 )
 
 if TYPE_CHECKING:
@@ -228,24 +234,35 @@ INFOSCHEMA_PAGENAME = "_camcops_information_schema_columns"
 # Export tasks from the back end
 # =============================================================================
 
-def print_export_queue(req: "CamcopsRequest",
-                       recipient_names: List[str] = None,
-                       all_recipients: bool = False,
-                       via_index: bool = True,
-                       pretty: bool = False) -> None:
+def print_export_queue(
+        req: "CamcopsRequest",
+        recipient_names: List[str] = None,
+        all_recipients: bool = False,
+        via_index: bool = True,
+        pretty: bool = False,
+        debug_show_fhir: bool = False,
+        debug_fhir_include_docs: bool = False) -> None:
     """
-    Called from the command line.
-
     Shows tasks that would be exported.
 
+    - Called from the command line.
+
     Args:
-        req: a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
-        recipient_names: list of export recipient names (as per the config
-            file)
-        all_recipients: use all recipients?
-        via_index: use the task index (faster)?
-        pretty: use ``str(task)`` not ``repr(task)`` (prettier, slower because
+        req:
+            a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
+        recipient_names:
+            list of export recipient names (as per the config file)
+        all_recipients:
+            use all recipients?
+        via_index:
+            use the task index (faster)?
+        pretty:
+            use ``str(task)`` not ``repr(task)`` (prettier, but slower because
             it has to query the patient)
+        debug_show_fhir:
+            Show FHIR output for each task, as JSON?
+        debug_fhir_include_docs:
+            (If debug_show_fhir.) Include document content? Large!
     """
     recipients = req.get_export_recipients(
         recipient_names=recipient_names,
@@ -264,6 +281,17 @@ def print_export_queue(req: "CamcopsRequest",
                 f"{recipient.recipient_name}: "
                 f"{str(task) if pretty else repr(task)}"
             )
+            if debug_show_fhir:
+                try:
+                    bundle = task.get_fhir_bundle(
+                        req, recipient,
+                        skip_docs_if_other_content=not debug_fhir_include_docs
+                    )
+                    bundle_str = json.dumps(bundle.as_json(),
+                                            indent=JSON_INDENT)
+                    log.info("FHIR output as JSON:\n{}", bundle_str)
+                except FhirExportException as e:
+                    log.info("Task has no non-document content:\n{}", e)
 
 
 def export(req: "CamcopsRequest",
@@ -272,12 +300,12 @@ def export(req: "CamcopsRequest",
            via_index: bool = True,
            schedule_via_backend: bool = False) -> None:
     """
-    Called from the command line.
-
     Exports all relevant tasks (pending incremental exports, or everything if
     applicable) for specified export recipients.
 
-    Obtains a file lock, then iterates through all recipients.
+    - Called from the command line, or from
+      :func:`camcops_server.cc_modules.celery.export_to_recipient_backend`.
+    - Calls :func:`export_whole_database` or :func:`export_tasks_individually`.
 
     Args:
         req: a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
@@ -294,10 +322,12 @@ def export(req: "CamcopsRequest",
         return
 
     for recipient in recipients:
-        log.info("Exporting to recipient: {}", recipient)
+        log.info("Exporting to recipient: {}", recipient.recipient_name)
         if recipient.using_db():
             if schedule_via_backend:
-                raise NotImplementedError()  # todo: implement whole-database export via Celery backend  # noqa
+                raise NotImplementedError(
+                    "Not yet implemented: whole-database export via Celery "
+                    "backend")  # todo: implement whole-database export via Celery backend  # noqa
             else:
                 export_whole_database(req, recipient, via_index=via_index)
         else:
@@ -305,6 +335,7 @@ def export(req: "CamcopsRequest",
             export_tasks_individually(
                 req, recipient,
                 via_index=via_index, schedule_via_backend=schedule_via_backend)
+        log.info("Finished exporting to {}", recipient.recipient_name)
 
 
 def export_whole_database(req: "CamcopsRequest",
@@ -313,15 +344,20 @@ def export_whole_database(req: "CamcopsRequest",
     """
     Exports to a database.
 
-    Holds a recipient-specific file lock in the process.
+    - Called by :func:`export`.
+    - Holds a recipient-specific "database" file lock in the process.
 
     Args:
-        req: a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
-        recipient: an :class:`camcops_server.cc_modules.cc_exportmodels.ExportRecipient`
-        via_index: use the task index (faster)?
-    """  # noqa
+        req:
+            a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
+        recipient:
+            an
+            :class:`camcops_server.cc_modules.cc_exportmodels.ExportRecipient`
+        via_index:
+            use the task index (faster)?
+    """
     cfg = req.config
-    lockfilename = cfg.get_export_lockfilename_db(
+    lockfilename = cfg.get_export_lockfilename_recipient_db(
         recipient_name=recipient.recipient_name)
     try:
         with lockfile.FileLock(lockfilename, timeout=0):  # doesn't wait
@@ -348,8 +384,11 @@ def export_whole_database(req: "CamcopsRequest",
             )
             dst_session.commit()
     except lockfile.AlreadyLocked:
-        log.warning("Export logfile {!r} already locked by another process; "
-                    "aborting", lockfilename)
+        log.warning(
+            "Export logfile {!r} already locked by another process; "
+            "aborting (another process is doing this work)", lockfilename)
+        # No need to retry by raising -- if someone else holds this lock, they
+        # are doing the work that we wanted to do.
 
 
 def export_tasks_individually(req: "CamcopsRequest",
@@ -359,15 +398,26 @@ def export_tasks_individually(req: "CamcopsRequest",
     """
     Exports all necessary tasks for a recipient.
 
+    - Called by :func:`export`.
+    - Calls :func:`export_task`, if ``schedule_via_backend`` is False.
+    - Schedules :func:``camcops_server.cc_modules.celery.export_task_backend``,
+      if ``schedule_via_backend`` is True, which calls :func:`export` in turn.
+
     Args:
-        req: a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
-        recipient: an :class:`camcops_server.cc_modules.cc_exportmodels.ExportRecipient`
-        via_index: use the task index (faster)?
-        schedule_via_backend: schedule jobs via the backend instead?
-    """  # noqa
+        req:
+            a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
+        recipient:
+            an
+            :class:`camcops_server.cc_modules.cc_exportmodels.ExportRecipient`
+        via_index:
+            use the task index (faster)?
+        schedule_via_backend:
+            schedule jobs via the backend instead?
+    """
     collection = get_collection_for_export(req, recipient, via_index=via_index)
+    n_tasks = 0
+    recipient_name = recipient.recipient_name
     if schedule_via_backend:
-        recipient_name = recipient.recipient_name
         for task_or_index in collection.gen_all_tasks_or_indexes():
             if isinstance(task_or_index, Task):
                 basetable = task_or_index.tablename
@@ -375,13 +425,16 @@ def export_tasks_individually(req: "CamcopsRequest",
             else:
                 basetable = task_or_index.task_table_name
                 task_pk = task_or_index.task_pk
-            log.info("Submitting background job to export task {}.{} to {}",
+            log.info("Scheduling job to export task {}.{} to {}",
                      basetable, task_pk, recipient_name)
             export_task_backend.delay(
                 recipient_name=recipient_name,
                 basetable=basetable,
                 task_pk=task_pk
             )
+            n_tasks += 1
+        log.info(f"Scheduled {n_tasks} background task exports to "
+                 f"{recipient_name}")
     else:
         for task in collection.gen_tasks_by_class():
             # Do NOT use this to check the working of export_task_backend():
@@ -389,6 +442,8 @@ def export_tasks_individually(req: "CamcopsRequest",
             # ... it will deadlock at the database (because we're already
             # within a query of some sort, I presume)
             export_task(req, recipient, task)
+            n_tasks += 1
+        log.info(f"Exported {n_tasks} tasks to {recipient_name}")
 
 
 def export_task(req: "CamcopsRequest",
@@ -397,11 +452,23 @@ def export_task(req: "CamcopsRequest",
     """
     Exports a single task, checking that it remains valid to do so.
 
+    - Called by :func:`export_tasks_individually` directly, or called via
+      :func:``camcops_server.cc_modules.celery.export_task_backend`` if
+      :func:`export_tasks_individually` requested that.
+    - Calls
+      :meth:`camcops_server.cc_modules.cc_exportmodels.ExportedTask.export`.
+    - For FHIR, holds a recipient-specific "FHIR" file lock during export.
+    - Always holds a recipient-and-task-specific file lock during export.
+
     Args:
-        req: a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
-        recipient: an :class:`camcops_server.cc_modules.cc_exportmodels.ExportRecipient`
-        task: a :class:`camcops_server.cc_modules.cc_task.Task`
-    """  # noqa
+        req:
+            a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
+        recipient:
+            an
+            :class:`camcops_server.cc_modules.cc_exportmodels.ExportRecipient`
+        task:
+            a :class:`camcops_server.cc_modules.cc_task.Task`
+    """
 
     # Double-check it's OK! Just in case, for example, an old backend task has
     # persisted, or someone's managed to get an iffy back-end request in some
@@ -411,14 +478,46 @@ def export_task(req: "CamcopsRequest",
         return
 
     cfg = req.config
-    lockfilename = cfg.get_export_lockfilename_task(
+    lockfilename = cfg.get_export_lockfilename_recipient_task(
         recipient_name=recipient.recipient_name,
         basetable=task.tablename,
         pk=task.pk,
     )
     dbsession = req.dbsession
-    try:
-        with lockfile.FileLock(lockfilename, timeout=0):  # doesn't wait
+    with ExitStack() as stack:
+
+        if recipient.using_fhir() and not recipient.fhir_concurrent:
+            # Some FHIR servers struggle with parallel processing, so we hold
+            # a lock to serialize them. See notes in cc_fhir.py.
+            #
+            # We always use the order (1) FHIR lockfile, (2) task lockfile, to
+            # avoid a deadlock.
+            #
+            # (Note that it is impossible that a non-FHIR task export grabs the
+            # second of these without the first, because the second lockfile is
+            # recipient-specific and the recipient details include the fact
+            # that it is a FHIR recipient.)
+            fhir_lockfilename = cfg.get_export_lockfilename_recipient_fhir(
+                recipient_name=recipient.recipient_name
+            )
+            try:
+                stack.enter_context(
+                    lockfile.FileLock(fhir_lockfilename,
+                                      timeout=jittered_delay_s())
+                    # waits for a while
+                )
+            except lockfile.AlreadyLocked:
+                log.warning(
+                    "Export logfile {!r} already locked by another process; "
+                    "will try again later", fhir_lockfilename)
+                raise
+                # We will reschedule via Celery; see "self.retry(...)" in
+                # celery.py
+
+        try:
+            stack.enter_context(
+                lockfile.FileLock(lockfilename, timeout=0)  # doesn't wait
+            )
             # We recheck the export status once we hold the lock, in case
             # multiple jobs are competing to export it.
             if ExportedTask.task_already_exported(
@@ -426,7 +525,7 @@ def export_task(req: "CamcopsRequest",
                     recipient_name=recipient.recipient_name,
                     basetable=task.tablename,
                     task_pk=task.pk):
-                log.info("Task {!r} already exported to recipient {!r}; "
+                log.info("Task {!r} already exported to recipient {}; "
                          "ignoring", task, recipient)
                 # Not a warning; it's normal to see these because it allows the
                 # client API to skip some checks for speed.
@@ -436,9 +535,10 @@ def export_task(req: "CamcopsRequest",
             dbsession.add(et)
             et.export(req)
             dbsession.commit()  # so the ExportedTask is visible to others ASAP
-    except lockfile.AlreadyLocked:
-        log.warning("Export logfile {!r} already locked by another process; "
-                    "aborting", lockfilename)
+        except lockfile.AlreadyLocked:
+            log.warning(
+                "Export logfile {!r} already locked by another process; "
+                "aborting (another process is doing this work)", lockfilename)
 
 
 # =============================================================================
@@ -454,13 +554,17 @@ def gen_audited_tasks_for_task_class(
     adding to an audit description. Used for user-triggered downloads.
 
     Args:
-        collection: a :class:`camcops_server.cc_modules.cc_taskcollection.TaskCollection`
-        cls: the task class to generate
-        audit_descriptions: list of strings to be modified
+        collection:
+            a
+            :class:`camcops_server.cc_modules.cc_taskcollection.TaskCollection`
+        cls:
+            the task class to generate
+        audit_descriptions:
+            list of strings to be modified
 
     Yields:
         :class:`camcops_server.cc_modules.cc_task.Task` objects
-    """  # noqa
+    """
     pklist = []  # type: List[int]
     for task in collection.tasks_for_task_class(cls):
         pklist.append(task.pk)
@@ -727,9 +831,7 @@ class TaskCollectionExporter(object):
         if email.sent:
             log.info(f"Research dump emailed to {email_to}")
         else:
-            log.error(
-                f"Failed to email research dump to {email_to}"
-            )
+            log.error(f"Failed to email research dump to {email_to}")
 
     def schedule_download(self) -> None:
         """
@@ -1146,9 +1248,13 @@ class UserDownloadFile(object):
                  req: "CamcopsRequest" = None) -> None:
         """
         Args:
-            filename: filename relative to ``directory``
-            directory: directory
-            req: a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
+            filename:
+                Filename, either absolute, or if ``directory`` is specified,
+                relative to ``directory``.
+            directory:
+                Directory. If specified, ``filename`` must be within it.
+            req:
+                a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
 
         Notes:
 
@@ -1160,16 +1266,29 @@ class UserDownloadFile(object):
           (https://en.wikipedia.org/wiki/Unix_time).
         """
         self.filename = filename
-        self.directory = directory
         self.permitted_lifespan_min = permitted_lifespan_min
         self.req = req
 
         self.basename = os.path.basename(filename)
         _, self.extension = os.path.splitext(filename)
         if directory:
-            self.fullpath = os.path.join(directory, filename)
+            # filename must be within the directory specified
+            self.directory = os.path.abspath(directory)
+            candidate_path = os.path.abspath(
+                os.path.join(self.directory, filename))
+            if os.path.commonpath([directory, candidate_path]) != directory:
+                # Filename is not within directory.
+                # This is dodgy -- someone may have passed a filename like
+                # "../../dangerous_dir/unsafe_content.txt"
+                self.fullpath = ""
+                # ... ensures that "exists" will be False.
+            else:
+                self.fullpath = candidate_path
         else:
+            # filename is treated as an absolute path
+            self.directory = ""
             self.fullpath = filename
+
         try:
             self.statinfo = os.stat(self.fullpath)
             self.exists = True
