@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_exportmodels.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_exportmodels.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -30,6 +31,7 @@ camcops_server/cc_modules/cc_exportmodels.py
 
 import logging
 import os
+import posixpath
 import socket
 import subprocess
 import sys
@@ -68,6 +70,7 @@ from sqlalchemy.sql.sqltypes import (
 from camcops_server.cc_modules.cc_constants import (
     ConfigParamExportRecipient,
     FileType,
+    UTF8,
 )
 from camcops_server.cc_modules.cc_email import Email
 from camcops_server.cc_modules.cc_exportrecipient import (
@@ -76,6 +79,10 @@ from camcops_server.cc_modules.cc_exportrecipient import (
 from camcops_server.cc_modules.cc_exportrecipientinfo import (
     ExportTransmissionMethod,
 )
+from camcops_server.cc_modules.cc_fhir import (
+    FhirExportException,
+    FhirTaskExporter,
+)
 from camcops_server.cc_modules.cc_filename import (
     change_filename_ext,
 )
@@ -112,7 +119,6 @@ log = BraceStyleAdapter(logging.getLogger(__name__))
 # =============================================================================
 
 DOS_NEWLINE = "\r\n"
-UTF8 = "utf8"
 
 
 # =============================================================================
@@ -151,9 +157,8 @@ def get_collection_for_export(req: "CamcopsRequest",
         export_recipient=recipient,
     )
     if debug:
-        log.critical(
-            "get_collection_for_export(): recipient={!r}, collection={!r}",
-            recipient, collection)
+        log.debug("get_collection_for_export(): recipient={!r}, "
+                  "collection={!r}", recipient, collection)
     return collection
 
 
@@ -255,9 +260,11 @@ class ExportedTask(Base):
 
     recipient = relationship(ExportRecipient)
 
-    hl7_messages = relationship("ExportedTaskHL7Message")
-    filegroups = relationship("ExportedTaskFileGroup")
     emails = relationship("ExportedTaskEmail")
+    fhir_exports = relationship("ExportedTaskFhir")
+    filegroups = relationship("ExportedTaskFileGroup")
+    hl7_messages = relationship("ExportedTaskHL7Message")
+    redcap_exports = relationship("ExportedTaskRedcap")
 
     def __init__(self,
                  recipient: ExportRecipient = None,
@@ -311,11 +318,8 @@ class ExportedTask(Base):
                 self._task = task_factory_no_security_checks(
                     dbsession, self.basetable, self.task_server_pk)
             except KeyError:
-                log.warning(
-                    "Failed to retrieve task for basetable={!r}, PK={!r}",
-                    self.basetable,
-                    self.task_server_pk
-                )
+                log.warning("Failed to retrieve task for basetable={!r}, "
+                            "PK={!r}", self.basetable, self.task_server_pk)
                 self._task = None
         return self._task
 
@@ -385,6 +389,12 @@ class ExportedTask(Base):
             dbsession.add(email)
             email.export_task(req)
 
+        elif transmission_method == ExportTransmissionMethod.FHIR:
+            efhir = ExportedTaskFhir(self)
+            dbsession.add(efhir)
+            dbsession.flush()
+            efhir.export_task(req)
+
         elif transmission_method == ExportTransmissionMethod.FILE:
             efg = ExportedTaskFileGroup(self)
             dbsession.add(efg)
@@ -402,6 +412,7 @@ class ExportedTask(Base):
             eredcap = ExportedTaskRedcap(self)
             dbsession.add(eredcap)
             eredcap.export_task(req)
+
         else:
             raise AssertionError("Bug: bad transmission_method")
 
@@ -666,7 +677,7 @@ class ExportedTaskHL7Message(Base):
 
         See
 
-        - http://python-hl7.readthedocs.org/en/latest/index.html
+        - https://python-hl7.readthedocs.org/en/latest/index.html
         """
         task = self.exported_task.task
         recipient = self.exported_task.recipient
@@ -696,12 +707,12 @@ class ExportedTaskHL7Message(Base):
         - Default MLLP/HL7 port is 2575
         - MLLP = minimum lower layer protocol
 
-          - http://www.cleo.com/support/byproduct/lexicom/usersguide/mllp_configuration.htm
-          - http://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.xhtml?search=hl7
+          - https://www.cleo.com/support/byproduct/lexicom/usersguide/mllp_configuration.htm
+          - https://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.xhtml?search=hl7
           - Essentially just a TCP socket with a minimal wrapper:
-            http://stackoverflow.com/questions/11126918
+            https://stackoverflow.com/questions/11126918
 
-        - http://python-hl7.readthedocs.org/en/latest/api.html; however,
+        - https://python-hl7.readthedocs.org/en/latest/api.html; however,
           we've modified that
         """  # noqa
         recipient = self.exported_task.recipient
@@ -748,6 +759,7 @@ class ExportedTaskHL7Message(Base):
 
     @staticmethod
     def ping_hl7_server(recipient: ExportRecipient) -> bool:
+        # noinspection HttpUrlsUsage
         """
         Performs a TCP/IP ping on our HL7 server; returns success. If we've
         already pinged successfully during this run, don't bother doing it
@@ -1164,3 +1176,112 @@ class ExportedTaskRedcap(Base):
             exported_task.succeed()
         except RedcapExportException as e:
             exported_task.abort(str(e))
+
+
+# =============================================================================
+# FHIR export
+# =============================================================================
+
+class ExportedTaskFhir(Base):
+    """
+    Represents an individual FHIR export.
+    """
+    __tablename__ = "_exported_task_fhir"
+
+    id = Column(
+        "id", Integer, primary_key=True, autoincrement=True,
+        comment="Arbitrary primary key"
+    )
+
+    exported_task_id = Column(
+        "exported_task_id", BigInteger, ForeignKey(ExportedTask.id),
+        nullable=False,
+        comment=f"FK to {ExportedTask.__tablename__}.{ExportedTask.id.name}"
+    )
+
+    exported_task = relationship(ExportedTask)
+
+    entries = relationship("ExportedTaskFhirEntry")
+
+    def __init__(self, exported_task: ExportedTask = None) -> None:
+        """
+        Args:
+            exported_task: :class:`ExportedTask` object
+        """
+        self.exported_task = exported_task
+
+    def export_task(self, req: "CamcopsRequest") -> None:
+        """
+        Exports the task to FHIR.
+
+        Args:
+            req: a :class:`camcops_server.cc_modules.cc_request.CamcopsRequest`
+        """
+        exported_task = self.exported_task
+
+        try:
+            exporter = FhirTaskExporter(req, self)
+            exporter.export_task()
+            exported_task.succeed()
+        except FhirExportException as e:
+            exported_task.abort(str(e))
+
+
+class ExportedTaskFhirEntry(Base):
+    """
+    Details of Patients, Questionnaires, QuestionnaireResponses exported to
+    a FHIR server for a single task.
+    """
+    __tablename__ = "_exported_task_fhir_entry"
+
+    id = Column(
+        "id", Integer, primary_key=True, autoincrement=True,
+        comment="Arbitrary primary key"
+    )
+
+    exported_task_fhir_id = Column(
+        "exported_task_fhir_id", Integer, ForeignKey(ExportedTaskFhir.id),
+        nullable=False,
+        comment="FK to {}.{}".format(ExportedTaskFhir.__tablename__,
+                                     ExportedTaskFhir.id.name)
+    )
+
+    etag = Column(
+        "etag", UnicodeText, comment="The ETag for the resource (if relevant)"
+    )
+
+    last_modified = Column(
+        "last_modified", DateTime,
+        comment="Server's date/time modified."
+    )
+
+    location = Column(
+        "location", UnicodeText,
+        comment="The location (if the operation returns a location)."
+    )
+
+    status = Column(
+        "status", UnicodeText,
+        comment="Status response code (text optional)."
+    )
+
+    # TODO: outcome?
+
+    exported_task_fhir = relationship(ExportedTaskFhir)
+
+    @property
+    def location_url(self) -> str:
+        """
+        Puts the FHIR server API URL together with the returned location, so
+        we can hyperlink to the resource.
+        """
+        if not self.location:
+            return ""
+        try:
+            api_url = self.exported_task_fhir.exported_task.recipient.fhir_api_url  # noqa
+        except Exception:
+            return ""
+        # Avoid urllib.parse.urljoin; it does complex (and for our purposes
+        # wrong) things. See
+        # https://stackoverflow.com/questions/10893374/python-confusions-with-urljoin
+        return posixpath.join(api_url, self.location)
