@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_exportrecipient.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_exportrecipient.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -91,13 +92,18 @@ class ExportRecipient(ExportRecipientInfo, Base):
     __tablename__ = "_export_recipients"
 
     IGNORE_FOR_EQ_ATTRNAMES = ExportRecipientInfo.IGNORE_FOR_EQ_ATTRNAMES + [
-        # Attribute names to ignore for equality comparison
+        # Attribute names to ignore for equality comparison (is one recipient
+        # record functionally equal to another?).
         "id",
         "current",
         "group_names",  # Python only
     ]
     NEEDS_RECOPYING_EACH_TIME_FROM_CONFIG_ATTRNAMES = [
+        # Fields representing sensitive information, not stored in the
+        # database. See also init_on_load() function.
         "email_host_password",
+        "fhir_app_secret",
+        "fhir_launch_token",
         "redcap_api_key",
     ]
 
@@ -373,6 +379,22 @@ class ExportRecipient(ExportRecipientInfo, Base):
         comment="(REDCap) File defining CamCOPS-to-REDCap field mapping"
     )
 
+    # -------------------------------------------------------------------------
+    # FHIR export
+    # -------------------------------------------------------------------------
+    fhir_api_url = Column(
+        "fhir_api_url", Text,
+        comment="(FHIR) FHIR API URL, pointing to the FHIR server"
+    )
+    fhir_app_id = Column(
+        "fhir_app_id", Text,
+        comment="(FHIR) FHIR app ID, identifying CamCOPS as the data source"
+    )
+    fhir_concurrent = Column(
+        "fhir_concurrent", Boolean, default=False, nullable=True,
+        comment="(FHIR) Server supports concurrency (parallel processing)?"
+    )
+
     def __init__(self, *args, **kwargs) -> None:
         """
         Creates a blank :class:`ExportRecipient` object.
@@ -394,10 +416,18 @@ class ExportRecipient(ExportRecipientInfo, Base):
         """
         Called when SQLAlchemy recreates an object; see
         https://docs.sqlalchemy.org/en/latest/orm/constructors.html.
+
+        Sets Python-only attributes.
+
+        See also IGNORE_FOR_EQ_ATTRNAMES,
+        NEEDS_RECOPYING_EACH_TIME_FROM_CONFIG_ATTRNAMES.
         """
-        # Python only:
         self.group_names = []  # type: List[str]
+
+        # Within NEEDS_RECOPYING_EACH_TIME_FROM_CONFIG_ATTRNAMES:
         self.email_host_password = ""
+        self.fhir_app_secret = ""
+        self.fhir_launch_token = None  # type: Optional[str]
         self.redcap_api_key = ""
 
     def get_attrnames(self) -> List[str]:
