@@ -1,11 +1,13 @@
 #!/usr/bin/env python
 
+# noinspection HttpUrlsUsage
 """
 camcops_server/cc_modules/cc_hl7.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,39 +22,39 @@ camcops_server/cc_modules/cc_hl7.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
-**Core HL7 functions, e.g. to build HL7 messages.**
+**Core HL7 functions, e.g. to build HL7 v2 messages.**
 
 General HL7 sources:
 
-- http://python-hl7.readthedocs.org/en/latest/
+- https://python-hl7.readthedocs.org/en/latest/
 - http://www.interfaceware.com/manual/v3gen_python_library_details.html
 - http://www.interfaceware.com/hl7_video_vault.html#how
 - http://www.interfaceware.com/hl7-standard/hl7-segments.html
-- http://www.hl7.org/special/committees/vocab/v26_appendix_a.pdf
-- http://www.ncbi.nlm.nih.gov/pmc/articles/PMC130066/
+- https://www.hl7.org/special/committees/vocab/v26_appendix_a.pdf
+- https://www.ncbi.nlm.nih.gov/pmc/articles/PMC130066/
 
 To consider
 
 - batched messages (HL7 batching protocol);
-  http://docs.oracle.com/cd/E23943_01/user.1111/e23486/app_hl7batching.htm
+  https://docs.oracle.com/cd/E23943_01/user.1111/e23486/app_hl7batching.htm
 - note: DG1 segment = diagnosis
 
 Basic HL7 message structure:
 
 - can package into HL7 2.X message as encapsulated PDF;
-  http://www.hl7standards.com/blog/2007/11/27/pdf-attachment-in-hl7-message/
+  https://www.hl7standards.com/blog/2007/11/27/pdf-attachment-in-hl7-message/
 - message ORU^R01
-  http://www.corepointhealth.com/resource-center/hl7-resources/hl7-messages
+  https://www.corepointhealth.com/resource-center/hl7-resources/hl7-messages
 - MESSAGES: http://www.interfaceware.com/hl7-standard/hl7-messages.html
 - OBX segment = observation/result segment;
-  http://www.corepointhealth.com/resource-center/hl7-resources/hl7-obx-segment;
+  https://www.corepointhealth.com/resource-center/hl7-resources/hl7-obx-segment;
   http://www.interfaceware.com/hl7-standard/hl7-segment-OBX.html
 - SEGMENTS:
-  http://www.corepointhealth.com/resource-center/hl7-resources/hl7-segments
+  https://www.corepointhealth.com/resource-center/hl7-resources/hl7-segments
 - ED field (= encapsulated data);
   http://www.interfaceware.com/hl7-standard/hl7-fields.html
 - base-64 encoding
@@ -113,13 +115,14 @@ ESCAPE_CHARACTER = "\\"
 # =============================================================================
 
 def get_mod11_checkdigit(strnum: str) -> str:
+    # noinspection HttpUrlsUsage
     """
     Input: string containing integer. Output: MOD11 check digit (string).
 
     See:
 
     - http://www.mexi.be/documents/hl7/ch200025.htm
-    - http://stackoverflow.com/questions/7006109
+    - https://stackoverflow.com/questions/7006109
     - http://www.pgrocer.net/Cis51/mod11.html
     """
     total = 0
@@ -146,14 +149,14 @@ def make_msh_segment(message_datetime: Pendulum,
     """
     Creates an HL7 message header (MSH) segment.
 
-    - MSH: http://www.hl7.org/documentcenter/public/wg/conf/HL7MSH.htm
+    - MSH: https://www.hl7.org/documentcenter/public/wg/conf/HL7MSH.htm
 
     - We're making an ORU^R01 message = unsolicited result.
 
       - ORU = Observational Report - Unsolicited
       - ORU^R01 = Unsolicited transmission of an observation message
-      - http://www.corepointhealth.com/resource-center/hl7-resources/hl7-oru-message
-      - http://www.hl7kit.com/joomla/index.php/hl7resources/examples/107-orur01
+      - https://www.corepointhealth.com/resource-center/hl7-resources/hl7-oru-message
+      - https://www.hl7kit.com/joomla/index.php/hl7resources/examples/107-orur01
     """  # noqa
 
     segment_id = "MSH"
@@ -219,11 +222,11 @@ def make_pid_segment(
     """
     Creates an HL7 patient identification (PID) segment.
 
-    - http://www.corepointhealth.com/resource-center/hl7-resources/hl7-pid-segment
-    - http://www.hl7.org/documentcenter/public/wg/conf/Msgadt.pdf (s5.4.8)
+    - https://www.corepointhealth.com/resource-center/hl7-resources/hl7-pid-segment
+    - https://www.hl7.org/documentcenter/public/wg/conf/Msgadt.pdf (s5.4.8)
 
     - ID numbers...
-      http://www.cdc.gov/vaccines/programs/iis/technical-guidance/downloads/hl7guide-1-4-2012-08.pdf
+      https://www.cdc.gov/vaccines/programs/iis/technical-guidance/downloads/hl7guide-1-4-2012-08.pdf
     """  # noqa
 
     patient_id_list = patient_id_list or []  # type: List[HL7PatientIdentifier]
@@ -328,14 +331,15 @@ def make_pid_segment(
 
 # noinspection PyUnusedLocal
 def make_obr_segment(task: "Task") -> hl7.Segment:
+    # noinspection HttpUrlsUsage
     """
     Creates an HL7 observation request (OBR) segment.
 
     - http://hl7reference.com/HL7%20Specifications%20ORM-ORU.PDF
     - Required in ORU^R01 message:
 
-      - http://www.corepointhealth.com/resource-center/hl7-resources/hl7-oru-message
-      - http://www.corepointhealth.com/resource-center/hl7-resources/hl7-obr-segment
+      - https://www.corepointhealth.com/resource-center/hl7-resources/hl7-oru-message
+      - https://www.corepointhealth.com/resource-center/hl7-resources/hl7-obr-segment
     """  # noqa
 
     segment_id = "OBR"
@@ -442,17 +446,18 @@ def make_obx_segment(req: "CamcopsRequest",
                      observation_datetime: Pendulum,
                      responsible_observer: str,
                      export_options: "TaskExportOptions") -> hl7.Segment:
+    # noinspection HttpUrlsUsage
     """
     Creates an HL7 observation result (OBX) segment.
 
     - http://www.hl7standards.com/blog/2006/10/18/how-do-i-send-a-binary-file-inside-of-an-hl7-message
     - http://www.hl7standards.com/blog/2007/11/27/pdf-attachment-in-hl7-message/
     - http://www.hl7standards.com/blog/2006/12/01/sending-images-or-formatted-documents-via-hl7-messaging/
-    - www.hl7.org/documentcenter/public/wg/ca/HL7ClmAttIG.PDF
+    - https://www.hl7.org/documentcenter/public/wg/ca/HL7ClmAttIG.PDF
     - type of data:
-      http://www.hl7.org/implement/standards/fhir/v2/0191/index.html
+      https://www.hl7.org/implement/standards/fhir/v2/0191/index.html
     - subtype of data:
-      http://www.hl7.org/implement/standards/fhir/v2/0291/index.html
+      https://www.hl7.org/implement/standards/fhir/v2/0291/index.html
     """  # noqa
 
     segment_id = "OBX"
@@ -562,6 +567,7 @@ def make_dg1_segment(set_id: int,
                      clinician_assigning_facility: str = "",
                      attestation_datetime: Pendulum = None) \
         -> hl7.Segment:
+    # noinspection HttpUrlsUsage
     """
     Creates an HL7 diagnosis (DG1) segment.
 
@@ -624,7 +630,7 @@ def make_dg1_segment(set_id: int,
     diagnosis_description = ""
     diagnosis_datetime = format_datetime(diagnosis_datetime,
                                          DateFormat.HL7_DATETIME)
-    if diagnosis_type not in ["A", "W", "F"]:
+    if diagnosis_type not in ("A", "W", "F"):
         raise AssertionError("make_dg1_segment: diagnosis_type invalid")
     major_diagnostic_category = ""
     diagnostic_related_group = ""
@@ -667,11 +673,11 @@ def make_dg1_segment(set_id: int,
         clinician_assigning_facility or "",
     ])
 
-    if diagnosis_classification not in ["C", "D", "M", "O", "R", "S", "T",
-                                        "I"]:
+    if diagnosis_classification not in ("C", "D", "M", "O", "R", "S", "T",
+                                        "I"):
         raise AssertionError(
             "make_dg1_segment: diagnosis_classification invalid")
-    if confidential_indicator not in ["Y", "N"]:
+    if confidential_indicator not in ("Y", "N"):
         raise AssertionError(
             "make_dg1_segment: confidential_indicator invalid")
     attestation_datetime = (
@@ -706,6 +712,7 @@ def make_dg1_segment(set_id: int,
 
 
 def escape_hl7_text(s: str) -> str:
+    # noinspection HttpUrlsUsage
     """
     Escapes HL7 special characters.
 
@@ -734,6 +741,7 @@ def escape_hl7_text(s: str) -> str:
 
 
 def msg_is_successful_ack(msg: hl7.Message) -> Tuple[bool, Optional[str]]:
+    # noinspection HttpUrlsUsage
     """
     Checks whether msg represents a successful acknowledgement message.
 
