@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_patient.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_patient.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -41,8 +42,15 @@ from cardinal_pythonlib.datetimefunc import (
     get_age,
     PotentialDatetimeType,
 )
+from cardinal_pythonlib.json.typing_helpers import JsonObjectType
 from cardinal_pythonlib.logs import BraceStyleAdapter
 import cardinal_pythonlib.rnc_web as ws
+from fhirclient.models.address import Address
+from fhirclient.models.contactpoint import ContactPoint
+from fhirclient.models.humanname import HumanName
+from fhirclient.models.fhirreference import FHIRReference
+from fhirclient.models.identifier import Identifier
+from fhirclient.models.patient import Patient as FhirPatient
 import hl7
 import pendulum
 from sqlalchemy.ext.declarative import declared_attr
@@ -59,15 +67,20 @@ from camcops_server.cc_modules.cc_audit import audit
 from camcops_server.cc_modules.cc_constants import (
     DateFormat,
     ERA_NOW,
+    FHIRConst as Fc,
     FP_ID_DESC,
     FP_ID_SHORT_DESC,
     FP_ID_NUM,
     SEX_FEMALE,
     SEX_MALE,
+    SEX_OTHER_UNSPECIFIED,
     TSV_PATIENT_FIELD_PREFIX,
 )
 from camcops_server.cc_modules.cc_db import GenericTabletRecordMixin
-from camcops_server.cc_modules.cc_device import Device
+from camcops_server.cc_modules.cc_fhir import (
+    fhir_pk_identifier,
+    make_fhir_bundle_entry,
+)
 from camcops_server.cc_modules.cc_hl7 import make_pid_segment
 from camcops_server.cc_modules.cc_html import answer
 from camcops_server.cc_modules.cc_simpleobjects import (
@@ -106,6 +119,7 @@ if TYPE_CHECKING:
     from camcops_server.cc_modules.cc_policy import TokenizedPolicy
     from camcops_server.cc_modules.cc_request import CamcopsRequest
     from camcops_server.cc_modules.cc_taskschedule import PatientTaskSchedule
+    from camcops_server.cc_modules.cc_user import User
 
 log = BraceStyleAdapter(logging.getLogger(__name__))
 
@@ -136,13 +150,13 @@ class Patient(GenericTabletRecordMixin, Base):
         index=True,
         identifies_patient=True, include_in_anon_staging_db=True,
         comment="Forename"
-    )
+    )  # type: Optional[str]
     surname = CamcopsColumn(
         "surname", PatientNameColType,
         index=True,
         identifies_patient=True, include_in_anon_staging_db=True,
         comment="Surname"
-    )
+    )  # type: Optional[str]
     dob = CamcopsColumn(
         "dob", sqltypes.Date,  # verified: merge_db handles this correctly
         index=True,
@@ -203,7 +217,9 @@ class Patient(GenericTabletRecordMixin, Base):
 
     task_schedules = relationship(
         "PatientTaskSchedule",
-        back_populates="patient")  # type: List[PatientTaskSchedule]
+        back_populates="patient",
+        cascade="all, delete"
+    )  # type: List[PatientTaskSchedule]
 
     # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
     # THE FOLLOWING ARE DEFUNCT, AND THE SERVER WORKS AROUND OLD TABLETS IN
@@ -420,9 +436,9 @@ class Patient(GenericTabletRecordMixin, Base):
 
         """
         # Same object?
-        # log.warning("self={}, other={}", self, other)
+        # log.debug("self={}, other={}", self, other)
         if self is other:
-            # log.warning("... same object; equal")
+            # log.debug("... same object; equal")
             return True
         # Same device/era/patient ID (client PK)? Test int before str for speed
         if (self.id == other.id and
@@ -431,15 +447,15 @@ class Patient(GenericTabletRecordMixin, Base):
                 self.id is not None and
                 self._device_id is not None and
                 self._era is not None):
-            # log.warning("... same device/era/id; equal")
+            # log.debug("... same device/era/id; equal")
             return True
         # Shared ID number?
         for sid in self.idnums:
             if sid in other.idnums:
-                # log.warning("... share idnum {}; equal", sid)
+                # log.debug("... share idnum {}; equal", sid)
                 return True
         # Otherwise...
-        # log.warning("... unequal")
+        # log.debug("... unequal")
         return False
 
     def __hash__(self) -> int:
@@ -628,12 +644,11 @@ class Patient(GenericTabletRecordMixin, Base):
 
     def get_surname_forename_upper(self) -> str:
         """
-        Get "SURNAME, FORENAME" in HTML-safe format, using "(UNKNOWN)" for
-        missing details.
+        Get "SURNAME, FORENAME", using "(UNKNOWN)" for missing details.
         """
         s = self.surname.upper() if self.surname else "(UNKNOWN)"
         f = self.forename.upper() if self.forename else "(UNKNOWN)"
-        return ws.webify(s + ", " + f)
+        return f"{s}, {f}"
 
     def get_dob_html(self, req: "CamcopsRequest", longform: bool) -> str:
         """
@@ -859,6 +874,121 @@ class Patient(GenericTabletRecordMixin, Base):
             patient_id_list=patient_id_tuple_list,
         )
 
+    # -------------------------------------------------------------------------
+    # FHIR
+    # -------------------------------------------------------------------------
+
+    def get_fhir_bundle_entry(self,
+                              req: "CamcopsRequest",
+                              recipient: "ExportRecipient") -> Dict[str, Any]:
+        """
+        Returns a dictionary, suitable for serializing to JSON, that
+        encapsulates patient identity information in a FHIR bundle.
+
+        See https://www.hl7.org/fhir/patient.html.
+        """
+        # The JSON objects we will build up:
+        patient_dict = {}  # type: JsonObjectType
+
+        # Name
+        if self.forename or self.surname:
+            name_dict = {}  # type: JsonObjectType
+            if self.forename:
+                name_dict[Fc.NAME_GIVEN] = [self.forename]
+            if self.surname:
+                name_dict[Fc.NAME_FAMILY] = self.surname
+            patient_dict[Fc.NAME] = [HumanName(jsondict=name_dict).as_json()]
+
+        # DOB
+        if self.dob:
+            patient_dict[Fc.BIRTHDATE] = format_datetime(
+                self.dob, DateFormat.FILENAME_DATE_ONLY)
+
+        # Sex/gender (should always be present, per client minimum ID policy)
+        if self.sex:
+            gender_lookup = {
+                SEX_FEMALE: Fc.GENDER_FEMALE,
+                SEX_MALE: Fc.GENDER_MALE,
+                SEX_OTHER_UNSPECIFIED: Fc.GENDER_OTHER,
+            }
+            patient_dict[Fc.GENDER] = gender_lookup.get(self.sex,
+                                                        Fc.GENDER_UNKNOWN)
+
+        # Address
+        if self.address:
+            patient_dict[Fc.ADDRESS] = [
+                Address(jsondict={
+                    Fc.ADDRESS_TEXT: self.address
+                }).as_json()
+            ]
+
+        # Email
+        if self.email:
+            patient_dict[Fc.TELECOM] = [
+                ContactPoint(jsondict={
+                    Fc.SYSTEM: Fc.TELECOM_SYSTEM_EMAIL,
+                    Fc.VALUE: self.email
+                }).as_json()
+            ]
+
+        # General practitioner (GP): via
+        # fhirclient.models.fhirreference.FHIRReference; too structured.
+
+        # ID numbers go here:
+        return make_fhir_bundle_entry(
+            resource_type_url=Fc.RESOURCE_TYPE_PATIENT,
+            identifier=self.get_fhir_identifier(req, recipient),
+            resource=FhirPatient(jsondict=patient_dict).as_json()
+        )
+
+    def get_fhir_identifier(self,
+                            req: "CamcopsRequest",
+                            recipient: "ExportRecipient") -> Identifier:
+        """
+        Returns a FHIR identifier for this patient, as a
+        :class:`fhirclient.models.identifier.Identifier` object.
+
+        This pairs a URL to our CamCOPS server indicating the ID number type
+        (as the "system") with the actual ID number (as the "value").
+
+        For debugging situations, it falls back to a default identifier (using
+        the PK on our CamCOPS server).
+        """
+        which_idnum = recipient.primary_idnum
+        try:
+            # For real exports, the fact that the patient does have an ID
+            # number of the right type will have been pre-verified.
+            if which_idnum is None:
+                raise AttributeError
+            idnum_object = self.get_idnum_object(which_idnum)
+            idnum_value = idnum_object.idnum_value  # may raise AttributeError
+            iddef = req.get_idnum_definition(which_idnum)
+            idnum_url = iddef.effective_fhir_id_system(req)
+            return Identifier(jsondict={
+                Fc.SYSTEM: idnum_url,
+                Fc.VALUE: str(idnum_value),
+            })
+        except AttributeError:
+            # We are probably in a debugging/drafting situation. Fall back to
+            # a default identifier.
+            return fhir_pk_identifier(
+                req, self.__tablename__, self.pk,
+                Fc.CAMCOPS_VALUE_PATIENT_WITHIN_TASK
+            )
+
+    def get_fhir_subject_ref(
+            self,
+            req: "CamcopsRequest",
+            recipient: "ExportRecipient") -> Dict:
+        """
+        Returns a FHIRReference (in JSON dict format) used to refer to this
+        patient as a "subject" of some other entry (like a questionnaire).
+        """
+        return FHIRReference(jsondict={
+            Fc.TYPE: Fc.RESOURCE_TYPE_PATIENT,
+            Fc.IDENTIFIER: self.get_fhir_identifier(req, recipient).as_json(),
+        }).as_json()
+
     # -------------------------------------------------------------------------
     # Database status
     # -------------------------------------------------------------------------
@@ -935,29 +1065,24 @@ class Patient(GenericTabletRecordMixin, Base):
         super().delete_with_dependants(req)
 
     # -------------------------------------------------------------------------
-    # Editing
+    # Permissions
     # -------------------------------------------------------------------------
 
-    def is_finalized(self) -> bool:
+    def user_may_view(self, user: "User") -> bool:
         """
-        Is the patient finalized (no longer available to be edited on the
-        client device), and therefore editable on the server?
+        May this user inspect patient details directly?
         """
-        if self._era == ERA_NOW:
-            # Not finalized; no editing on server
-            return False
-        return True
-
-    def created_on_server(self, req: "CamcopsRequest") -> bool:
-        server_device = Device.get_server_device(req.dbsession)
-
-        return (self._era == ERA_NOW and
-                self._device_id == server_device.id)
+        return self._group_id in user.ids_of_groups_user_may_see
 
     def user_may_edit(self, req: "CamcopsRequest") -> bool:
         """
         Does the current user have permission to edit this patient?
         """
+        if self.created_on_server(req):
+            # Anyone in the group with the right permission
+            return req.user.may_manage_patients_in_group(self._group_id)
+
+        # Finalized patient: Need to be group administrator
         return req.user.may_administer_group(self._group_id)
 
     # --------------------------------------------------------------------------
