@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_policy.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_policy.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -1084,7 +1085,7 @@ class TokenizedPolicy(object):
                       "beyond end of policy; bad policy")
             return Q_ERROR, start
         token = tokens[start]
-        if token in [TK_RPAREN, TK_AND, TK_OR]:
+        if token in (TK_RPAREN, TK_AND, TK_OR):
             log.debug("_content_chunk_value(): "
                       "chunk starts with ), AND, or OR; bad policy")
             return Q_ERROR, start
@@ -1145,7 +1146,7 @@ class TokenizedPolicy(object):
             log.debug("_op(): beyond end of policy")
             return None, start
         token = policy[start]
-        if token in [TK_AND, TK_OR]:
+        if token in (TK_AND, TK_OR):
             return token, start + 1
         else:
             log.debug("_op(): not an operator; bad policy")
