@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_pyramid.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_pyramid.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -41,6 +42,7 @@ from urllib.parse import urlencode
 # from cardinal_pythonlib.debugging import get_caller_stack_info
 from cardinal_pythonlib.logs import BraceStyleAdapter
 from cardinal_pythonlib.wsgi.constants import WsgiEnvVar
+from mako.filters import html_escape
 from mako.lookup import TemplateLookup
 from paginate import Page
 from pyramid.authentication import IAuthenticationPolicy
@@ -56,7 +58,7 @@ from pyramid.security import (
     Everyone,
     PermitsResult,
 )
-from pyramid.session import SignedCookieSessionFactory
+from pyramid.session import JSONSerializer, SignedCookieSessionFactory
 from pyramid_mako import (
     MakoLookupTemplateRenderer,
     MakoRendererFactory,
@@ -77,6 +79,7 @@ if TYPE_CHECKING:
 
 log = BraceStyleAdapter(logging.getLogger(__name__))
 
+
 # =============================================================================
 # Debugging options
 # =============================================================================
@@ -129,14 +132,6 @@ class FormAction(object):
     REFRESH_TASKS = 'refresh_tasks'
 
 
-class RequestMethod(object):
-    """
-    Constants to distinguish HTTP GET from HTTP POST requests.
-    """
-    GET = "GET"
-    POST = "POST"
-
-
 class ViewParam(object):
     """
     View parameter constants.
@@ -154,10 +149,13 @@ class ViewParam(object):
     ADDRESS = "address"
     ADD_SPECIAL_NOTE = "add_special_note"
     ADMIN = "admin"
+    ADVANCED = "advanced"
     AGE_MINIMUM = "age_minimum"
     AGE_MAXIMUM = "age_maximum"
     ALL_TASKS = "all_tasks"
     ANONYMISE = "anonymise"
+    BACK_TASK_TABLENAME = "back_task_tablename"
+    BACK_TASK_SERVER_PK = "back_task_server_pk"
     CLINICIAN_CONFIRMATION = "clinician_confirmation"
     CSRF_TOKEN = "csrf_token"
     DATABASE_TITLE = "database_title"
@@ -168,15 +166,21 @@ class ViewParam(object):
     DIALECT = "dialect"
     DIAGNOSES_INCLUSION = "diagnoses_inclusion"
     DIAGNOSES_EXCLUSION = "diagnoses_exclusion"
+    DISABLE_MFA = "disable_mfa"
     DUMP_METHOD = "dump_method"
     DOB = "dob"
     DUE_FROM = "due_from"
     DUE_WITHIN = "due_within"
     EMAIL = "email"
+    EMAIL_BCC = "email_bcc"
+    EMAIL_BODY = "email_body"
+    EMAIL_CC = "email_cc"
+    EMAIL_FROM = "email_from"
     EMAIL_SUBJECT = "email_subject"
     EMAIL_TEMPLATE = "email_template"
     END_DATETIME = "end_datetime"
     INCLUDE_AUTO_GENERATED = "include_auto_generated"
+    FHIR_ID_SYSTEM = "fhir_id_system"
     FILENAME = "filename"
     FINALIZE_POLICY = "finalize_policy"
     FORENAME = "forename"
@@ -202,22 +206,28 @@ class ViewParam(object):
     MANUAL = "manual"
     MAY_ADD_NOTES = "may_add_notes"
     MAY_DUMP_DATA = "may_dump_data"
+    MAY_EMAIL_PATIENTS = "may_email_patients"
+    MAY_MANAGE_PATIENTS = "may_manage_patients"
     MAY_REGISTER_DEVICES = "may_register_devices"
     MAY_RUN_REPORTS = "may_run_reports"
     MAY_UPLOAD = "may_upload"
     MAY_USE_WEBVIEWER = "may_use_webviewer"
+    MFA_SECRET_KEY = "mfa_secret_key"
+    MFA_METHOD = "mfa_method"
     MUST_CHANGE_PASSWORD = "must_change_password"
     NAME = "name"
     NOTE = "note"
     NOTE_ID = "note_id"
     NEW_PASSWORD = "new_password"
     OLD_PASSWORD = "old_password"
+    ONE_TIME_PASSWORD = "one_time_password"
     OTHER = "other"
     COMPLETE_ONLY = "complete_only"
     PAGE = "page"
     PASSWORD = "password"
     PATIENT_ID_PER_ROW = "patient_id_per_row"
     PATIENT_TASK_SCHEDULE_ID = "patient_task_schedule_id"
+    PHONE_NUMBER = "phone_number"
     RECIPIENT_NAME = "recipient_name"
     REDIRECT_URL = "redirect_url"
     REPORT_ID = "report_id"
@@ -268,6 +278,7 @@ class ViewArg(object):
     IMMEDIATELY = "immediately"
 
     # Output types
+    FHIRJSON = "fhirjson"
     HTML = "html"
     ODS = "ods"
     PDF = "pdf"
@@ -286,6 +297,21 @@ class ViewArg(object):
     USE_SESSION_FILTER = "use_session_filter"
 
 
+# =============================================================================
+# Flash message queues
+# =============================================================================
+
+class FlashQueue:
+    """
+    Predefined flash (alert) message queues for Bootstrap; see
+    https://getbootstrap.com/docs/3.3/components/#alerts.
+    """
+    SUCCESS = "success"
+    INFO = "info"
+    WARNING = "warning"
+    DANGER = "danger"
+
+
 # =============================================================================
 # Templates
 # =============================================================================
@@ -317,7 +343,13 @@ MAKO_LOOKUP = TemplateLookup(
     # uses those variables.
 
     # -------------------------------------------------------------------------
-    # TEMPLATE CACHING
+    # Template default filtering
+    # -------------------------------------------------------------------------
+
+    default_filters=["h"],
+
+    # -------------------------------------------------------------------------
+    # Template caching
     # -------------------------------------------------------------------------
     # http://dogpilecache.readthedocs.io/en/latest/api.html#module-dogpile.cache.plugins.mako_cache  # noqa
     # http://docs.makotemplates.org/en/latest/caching.html#cache-arguments
@@ -485,7 +517,7 @@ class CamcopsMakoLookupTemplateRenderer(MakoLookupTemplateRenderer):
             log.debug("spec: {!r}", self.spec)
             log.debug("value: {}", pprint.pformat(value))
             log.debug("system: {}", pprint.pformat(system))
-        # log.critical("\n{}", "\n    ".join(get_caller_stack_info()))
+            # log.debug("\n{}", "\n    ".join(get_caller_stack_info()))
 
         # ---------------------------------------------------------------------
         # RNC extra values:
@@ -717,14 +749,25 @@ class Routes(object):
     EDIT_GROUP = "edit_group"
     EDIT_ID_DEFINITION = "edit_id_definition"
     EDIT_FINALIZED_PATIENT = "edit_finalized_patient"
+    EDIT_OTHER_USER_MFA = "edit_other_user_mfa"
+    EDIT_OWN_USER_MFA = "edit_own_user_mfa"
     EDIT_SERVER_CREATED_PATIENT = "edit_server_created_patient"
     EDIT_SERVER_SETTINGS = "edit_server_settings"
     EDIT_TASK_SCHEDULE = "edit_task_schedule"
     EDIT_TASK_SCHEDULE_ITEM = "edit_task_schedule_item"
     EDIT_USER = "edit_user"
+    EDIT_USER_AUTHENTICATION = "edit_user_authentication"
     EDIT_USER_GROUP_MEMBERSHIP = "edit_user_group_membership"
     ERASE_TASK_LEAVING_PLACEHOLDER = "erase_task_leaving_placeholder"
     ERASE_TASK_ENTIRELY = "erase_task_entirely"
+    FHIR_CONDITION = "fhir_condition"
+    FHIR_DOCUMENT_REFERENCE = "fhir_document_reference"
+    FHIR_OBSERVATION = "fhir_observation"
+    FHIR_PATIENT_ID_SYSTEM = "fhir_patient_id_system"
+    FHIR_PRACTITIONER = "fhir_practitioner"
+    FHIR_QUESTIONNAIRE_SYSTEM = "fhir_questionnaire"
+    FHIR_QUESTIONNAIRE_RESPONSE = "fhir_questionnaire_response"
+    FHIR_TABLENAME_PK_ID = "fhir_tablename_pk_id"
     FORCIBLY_FINALIZE = "forcibly_finalize"
     HOME = "home"
     LOGIN = "login"
@@ -738,14 +781,20 @@ class Routes(object):
     OFFER_BASIC_DUMP = "offer_basic_dump"
     REPORT = "report"
     REPORTS_MENU = "reports_menu"
+    SEND_EMAIL_FROM_PATIENT_LIST = "send_email_from_patient_list"
+    SEND_EMAIL_FROM_PATIENT_TASK_SCHEDULE = "send_email_from_patient_task_schedule"  # noqa: E501
     SET_FILTERS = "set_filters"
     SET_OTHER_USER_UPLOAD_GROUP = "set_other_user_upload_group"
     SET_OWN_USER_UPLOAD_GROUP = "set_user_upload_group"
     SQL_DUMP = "sql_dump"
     TASK = "task"
+    TASK_DETAILS = "task_details"
+    TASK_LIST = "task_list"
+    TEST_NHS_NUMBERS = "test_nhs_numbers"
     TESTPAGE_PRIVATE_1 = "testpage_private_1"
     TESTPAGE_PRIVATE_2 = "testpage_private_2"
     TESTPAGE_PRIVATE_3 = "testpage_private_3"
+    TESTPAGE_PRIVATE_4 = "testpage_private_4"
     TESTPAGE_PUBLIC_1 = "testpage_public_1"
     TRACKER = "tracker"
     UNLOCK_USER = "unlock_user"
@@ -757,8 +806,11 @@ class Routes(object):
     VIEW_EXPORTED_TASK = "view_exported_task"
     VIEW_EXPORTED_TASK_LIST = "view_exported_task_list"
     VIEW_EXPORTED_TASK_EMAIL = "view_exported_task_email"
+    VIEW_EXPORTED_TASK_FHIR = "view_exported_task_fhir"
+    VIEW_EXPORTED_TASK_FHIR_ENTRY = "view_exported_task_fhir_entry"
     VIEW_EXPORTED_TASK_FILE_GROUP = "view_exported_task_file_group"
     VIEW_EXPORTED_TASK_HL7_MESSAGE = "view_exported_task_hl7_message"
+    VIEW_EXPORTED_TASK_REDCAP = "view_exported_task_redcap"
     VIEW_GROUPS = "view_groups"
     VIEW_ID_DEFINITIONS = "view_id_definitions"
     VIEW_OWN_USER_INFO = "view_own_user_info"
@@ -778,26 +830,80 @@ class RoutePath(object):
     Class to hold a route/path pair.
 
     - Pyramid route names are just strings used internally for convenience.
+
     - Pyramid URL paths are URL fragments, like ``'/thing'``, and can contain
       placeholders, like ``'/thing/{bork_id}'``, which will result in the
       ``request.matchdict`` object containing a ``'bork_id'`` key. Those can be
       further constrained by regular expressions, like
-      ``'/thing/{bork_id:\d+}'`` to restrict to digits.
+      ``'/thing/{bork_id:\d+}'`` to restrict to digits. See
+      https://docs.pylonsproject.org/projects/pyramid/en/latest/narr/urldispatch.html
 
-    """
+    """  # noqa
     def __init__(self, route: str, path: str = "",
-                 ignore_in_all_routes: bool = False) -> None:
+                 ignore_in_all_routes: bool = False,
+                 pregenerator: Callable = None) -> None:
         self.route = route
         self.path = path or "/" + route
         self.ignore_in_all_routes = ignore_in_all_routes
+        self.pregenerator = pregenerator
 
 
 MASTER_ROUTE_WEBVIEW = "/"
 MASTER_ROUTE_CLIENT_API = "/api"
 MASTER_ROUTE_CLIENT_API_ALIAS = "/database"
+
 STATIC_CAMCOPS_PACKAGE_PATH = "camcops_server.static:"
 # ... the "static" package (directory with __init__.py) within the
 # "camcops_server" owning package
+STATIC_BOOTSTRAP_ICONS_PATH = (STATIC_CAMCOPS_PACKAGE_PATH +
+                               "bootstrap-icons-1.7.0")
+
+
+# noinspection PyUnusedLocal
+def pregen_for_fhir(request: Request,
+                    elements: Tuple,
+                    kw: Dict) -> Tuple:
+    """
+    Pyramid pregenerator, to pre-populate an optional URL keyword (with an
+    empty string, as it happens). See
+
+    - https://stackoverflow.com/questions/42193305/optional-url-parameter-on-pyramid-route
+    - https://docs.pylonsproject.org/projects/pyramid/en/latest/api/config.html
+    - https://docs.pylonsproject.org/projects/pyramid/en/latest/api/interfaces.html#pyramid.interfaces.IRoutePregenerator
+    """  # noqa
+    kw.setdefault("fhirvalue_with_bar", "")
+    return elements, kw
+
+
+def _mk_fhir_optional_value_suffix_route(route: str,
+                                         path: str = "") -> RoutePath:
+    path = path or "/" + route
+    path_with_optional_value = path + r"{fhirvalue_with_bar:(\|[\w\d/\.]+)?}"
+    # ... allow, optionally, a bar followed by one or more word, digit,
+    # forward slash, or period characters.
+    # This allows FHIR identifier suffixes like path|table/2.4.11
+    return RoutePath(
+        route,
+        path_with_optional_value,
+        pregenerator=pregen_for_fhir
+    )
+
+
+def _mk_fhir_tablename_route(route: str) -> RoutePath:
+    return _mk_fhir_optional_value_suffix_route(
+        route,
+        f"/{route}"
+        rf"/{{{ViewParam.TABLE_NAME}:\w+}}"
+    )
+
+
+def _mk_fhir_tablename_pk_route(route: str) -> RoutePath:
+    return _mk_fhir_optional_value_suffix_route(
+        route,
+        f"/{route}"
+        rf"/{{{ViewParam.TABLE_NAME}:\w+}}"
+        rf"/{{{ViewParam.SERVER_PK}:\d+}}"
+    )
 
 
 class RouteCollection(object):
@@ -840,9 +946,7 @@ class RouteCollection(object):
     DELETE_GROUP = RoutePath(Routes.DELETE_GROUP)
     DELETE_ID_DEFINITION = RoutePath(Routes.DELETE_ID_DEFINITION)
     DELETE_PATIENT = RoutePath(Routes.DELETE_PATIENT)
-    DELETE_SERVER_CREATED_PATIENT = RoutePath(
-        Routes.DELETE_SERVER_CREATED_PATIENT
-    )
+    DELETE_SERVER_CREATED_PATIENT = RoutePath(Routes.DELETE_SERVER_CREATED_PATIENT)  # noqa: E501
     DELETE_SPECIAL_NOTE = RoutePath(Routes.DELETE_SPECIAL_NOTE)
     DELETE_TASK_SCHEDULE = RoutePath(Routes.DELETE_TASK_SCHEDULE)
     DELETE_TASK_SCHEDULE_ITEM = RoutePath(Routes.DELETE_TASK_SCHEDULE_ITEM)
@@ -853,14 +957,45 @@ class RouteCollection(object):
     EDIT_GROUP = RoutePath(Routes.EDIT_GROUP)
     EDIT_ID_DEFINITION = RoutePath(Routes.EDIT_ID_DEFINITION)
     EDIT_FINALIZED_PATIENT = RoutePath(Routes.EDIT_FINALIZED_PATIENT)
+    EDIT_OTHER_USER_MFA = RoutePath(Routes.EDIT_OTHER_USER_MFA)
+    EDIT_OWN_USER_MFA = RoutePath(Routes.EDIT_OWN_USER_MFA)
     EDIT_SERVER_CREATED_PATIENT = RoutePath(Routes.EDIT_SERVER_CREATED_PATIENT)
     EDIT_SERVER_SETTINGS = RoutePath(Routes.EDIT_SERVER_SETTINGS)
     EDIT_TASK_SCHEDULE = RoutePath(Routes.EDIT_TASK_SCHEDULE)
     EDIT_TASK_SCHEDULE_ITEM = RoutePath(Routes.EDIT_TASK_SCHEDULE_ITEM)
     EDIT_USER = RoutePath(Routes.EDIT_USER)
+    EDIT_USER_AUTHENTICATION = RoutePath(Routes.EDIT_USER_AUTHENTICATION)
     EDIT_USER_GROUP_MEMBERSHIP = RoutePath(Routes.EDIT_USER_GROUP_MEMBERSHIP)
-    ERASE_TASK_LEAVING_PLACEHOLDER = RoutePath(Routes.ERASE_TASK_LEAVING_PLACEHOLDER)  # noqa
+    ERASE_TASK_LEAVING_PLACEHOLDER = RoutePath(Routes.ERASE_TASK_LEAVING_PLACEHOLDER)  # noqa: E501
     ERASE_TASK_ENTIRELY = RoutePath(Routes.ERASE_TASK_ENTIRELY)
+
+    FHIR_CONDITION = _mk_fhir_tablename_pk_route(
+        Routes.FHIR_CONDITION,
+    )
+    FHIR_DOCUMENT_REFERENCE = _mk_fhir_tablename_pk_route(
+        Routes.FHIR_DOCUMENT_REFERENCE
+    )
+    FHIR_OBSERVATION = _mk_fhir_tablename_pk_route(
+        Routes.FHIR_OBSERVATION
+    )
+    FHIR_PATIENT_ID_SYSTEM = _mk_fhir_optional_value_suffix_route(
+        Routes.FHIR_PATIENT_ID_SYSTEM,
+        f"/{Routes.FHIR_PATIENT_ID_SYSTEM}"
+        rf"/{{{ViewParam.WHICH_IDNUM}:\d+}}"
+    )
+    FHIR_PRACTITIONER = _mk_fhir_tablename_pk_route(
+        Routes.FHIR_PRACTITIONER
+    )
+    FHIR_QUESTIONNAIRE_SYSTEM = _mk_fhir_optional_value_suffix_route(
+        Routes.FHIR_QUESTIONNAIRE_SYSTEM
+    )
+    FHIR_QUESTIONNAIRE_RESPONSE = _mk_fhir_tablename_pk_route(
+        Routes.FHIR_QUESTIONNAIRE_RESPONSE
+    )
+    FHIR_TABLENAME_PK_ID = _mk_fhir_tablename_pk_route(
+        Routes.FHIR_TABLENAME_PK_ID
+    )
+
     FORCIBLY_FINALIZE = RoutePath(Routes.FORCIBLY_FINALIZE)
     HOME = RoutePath(Routes.HOME, MASTER_ROUTE_WEBVIEW)  # mounted at "/"
     LOGIN = RoutePath(Routes.LOGIN)
@@ -873,14 +1008,23 @@ class RouteCollection(object):
     OFFER_BASIC_DUMP = RoutePath(Routes.OFFER_BASIC_DUMP)
     REPORT = RoutePath(Routes.REPORT)
     REPORTS_MENU = RoutePath(Routes.REPORTS_MENU)
+    SEND_EMAIL_FROM_PATIENT_LIST = RoutePath(Routes.SEND_EMAIL_FROM_PATIENT_LIST)  # noqa: E501
+    SEND_EMAIL_FROM_PATIENT_TASK_SCHEDULE = RoutePath(Routes.SEND_EMAIL_FROM_PATIENT_TASK_SCHEDULE)  # noqa: E501
     SET_FILTERS = RoutePath(Routes.SET_FILTERS)
     SET_OTHER_USER_UPLOAD_GROUP = RoutePath(Routes.SET_OTHER_USER_UPLOAD_GROUP)
     SET_OWN_USER_UPLOAD_GROUP = RoutePath(Routes.SET_OWN_USER_UPLOAD_GROUP)
     SQL_DUMP = RoutePath(Routes.SQL_DUMP)
     TASK = RoutePath(Routes.TASK)
+    TASK_DETAILS = RoutePath(
+        Routes.TASK_DETAILS,
+        rf"/{Routes.TASK_DETAILS}/{{{ViewParam.TABLE_NAME}}}"
+    )
+    TASK_LIST = RoutePath(Routes.TASK_LIST)
+    TEST_NHS_NUMBERS = RoutePath(Routes.TEST_NHS_NUMBERS)
     TESTPAGE_PRIVATE_1 = RoutePath(Routes.TESTPAGE_PRIVATE_1)
     TESTPAGE_PRIVATE_2 = RoutePath(Routes.TESTPAGE_PRIVATE_2)
     TESTPAGE_PRIVATE_3 = RoutePath(Routes.TESTPAGE_PRIVATE_3)
+    TESTPAGE_PRIVATE_4 = RoutePath(Routes.TESTPAGE_PRIVATE_4)
     TESTPAGE_PUBLIC_1 = RoutePath(Routes.TESTPAGE_PUBLIC_1)
     TRACKER = RoutePath(Routes.TRACKER)
     UNLOCK_USER = RoutePath(Routes.UNLOCK_USER)
@@ -892,8 +1036,11 @@ class RouteCollection(object):
     VIEW_EXPORTED_TASK = RoutePath(Routes.VIEW_EXPORTED_TASK)
     VIEW_EXPORTED_TASK_LIST = RoutePath(Routes.VIEW_EXPORTED_TASK_LIST)
     VIEW_EXPORTED_TASK_EMAIL = RoutePath(Routes.VIEW_EXPORTED_TASK_EMAIL)
+    VIEW_EXPORTED_TASK_FHIR = RoutePath(Routes.VIEW_EXPORTED_TASK_FHIR)
+    VIEW_EXPORTED_TASK_FHIR_ENTRY = RoutePath(Routes.VIEW_EXPORTED_TASK_FHIR_ENTRY)  # noqa
     VIEW_EXPORTED_TASK_FILE_GROUP = RoutePath(Routes.VIEW_EXPORTED_TASK_FILE_GROUP)  # noqa
     VIEW_EXPORTED_TASK_HL7_MESSAGE = RoutePath(Routes.VIEW_EXPORTED_TASK_HL7_MESSAGE)  # noqa
+    VIEW_EXPORTED_TASK_REDCAP = RoutePath(Routes.VIEW_EXPORTED_TASK_REDCAP)
     VIEW_GROUPS = RoutePath(Routes.VIEW_GROUPS)
     VIEW_ID_DEFINITIONS = RoutePath(Routes.VIEW_ID_DEFINITIONS)
     VIEW_OWN_USER_INFO = RoutePath(Routes.VIEW_OWN_USER_INFO)
@@ -985,7 +1132,11 @@ def get_session_factory() -> Callable[["CamcopsRequest"], ISession]:
             timeout=None,  # we handle timeouts at the database level instead
             reissue_time=0,  # default; reissue cookie at every request
             set_on_exception=True,  # (default) cookie even if exception raised
-            serializer=None,  # (default) use pyramid.session.PickleSerializer
+            serializer=JSONSerializer(),
+            # ... pyramid.session.PickleSerializer was the default but is
+            # deprecated as of Pyramid 1.9; the default is
+            # pyramid.session.JSONSerializer as of Pyramid 2.0.
+
             # As max_age and expires are left at their default of None, these
             # are session cookies.
         )
@@ -1006,9 +1157,12 @@ class Permission(object):
     - For "logged in", use ``pyramid.security.Authenticated``
     """
     GROUPADMIN = "groupadmin"
-    HAPPY = "happy"  # logged in, can use webview, no need to change p/w, agreed to terms  # noqa
+    HAPPY = "happy"
+    # ... logged in, can use webview, no need to change p/w, agreed to terms,
+    # a valid MFA method has been set.
     MUST_AGREE_TERMS = "must_agree_terms"
     MUST_CHANGE_PASSWORD = "must_change_password"
+    MUST_SET_MFA = "must_set_mfa"
     SUPERUSER = "superuser"
 
 
@@ -1061,6 +1215,8 @@ class CamcopsAuthenticationPolicy(object):
                     principals.append(Permission.MUST_CHANGE_PASSWORD)
                 elif user.must_agree_terms:
                     principals.append(Permission.MUST_AGREE_TERMS)
+                elif user.must_set_mfa_method(request):
+                    principals.append(Permission.MUST_SET_MFA)
                 else:
                     principals.append(Permission.HAPPY)
             if user.superuser:
@@ -1107,6 +1263,260 @@ class CamcopsAuthorizationPolicy(object):
         raise NotImplementedError()  # don't care about this method
 
 
+# =============================================================================
+# Icons
+# =============================================================================
+
+def icon_html(icon: str,
+              alt: str,
+              url: str = None,
+              extra_classes: List[str] = None,
+              extra_styles: List[str] = None,
+              escape_alt: bool = True) -> str:
+    """
+    Instantiates a Bootstrap icon, usually with a hyperlink. Returns
+    rendered HTML.
+
+    Args:
+        icon:
+            Icon name, without ".svg" extension (or "bi-" prefix!).
+        alt:
+            Alternative text for image.
+        url:
+            Optional URL of hyperlink.
+        extra_classes:
+            Optional extra CSS classes for the icon.
+        extra_styles:
+            Optional extra CSS styles for the icon (each looks like:
+            "color: blue").
+        escape_alt:
+            HTML-escape the alt text? Default is True.
+    """
+    # There are several ways to do this, such as via <img> tags, or via
+    # web fonts.
+    # We include bootstrap-icons.css (via base_web.mako), because that
+    # allows the best resizing (relative to font size) and styling.
+    # See:
+    # - https://icons.getbootstrap.com/#usage
+    # - http://johna.compoutpost.com/blog/1189/how-to-use-the-new-bootstrap-icons-v1-2-web-font/  # noqa
+    if escape_alt:
+        alt = html_escape(alt)
+    i_components = [
+        'role="img"',
+        f'aria-label="{alt}"'
+    ]
+    css_classes = [f"bi-{icon}"]  # bi = Bootstrap icon
+    if extra_classes:
+        css_classes += extra_classes
+    class_str = " ".join(css_classes)
+    i_components.append(f'class="{class_str}"')
+    if extra_styles:
+        style_str = "; ".join(extra_styles)
+        i_components.append(f'style="{style_str}"')
+    image = f'<i {" ".join(i_components)}></i>'
+    if url:
+        return f'<a href="{url}">{image}</a>'
+    else:
+        return image
+
+
+def icon_text(icon: str,
+              text: str,
+              url: str = None,
+              alt: str = None,
+              extra_icon_classes: List[str] = None,
+              extra_icon_styles: List[str] = None,
+              extra_a_classes: List[str] = None,
+              extra_a_styles: List[str] = None,
+              escape_alt: bool = True,
+              escape_text: bool = True,
+              hyperlink_together: bool = False) -> str:
+    """
+    Provide an icon and accompanying text. Usually, both are hyperlinked
+    (to the same destination URL). Returns rendered HTML.
+
+    Args:
+        icon:
+            Icon name, without ".svg" extension.
+        url:
+            Optional URL of hyperlink.
+        alt:
+            Alternative text for image. Will default to the main text.
+        text:
+            Main text to display.
+        extra_icon_classes:
+            Optional extra CSS classes for the icon.
+        extra_icon_styles:
+            Optional extra CSS styles for the icon (each looks like:
+            "color: blue").
+        extra_a_classes:
+            Optional extra CSS classes for the <a> element.
+        extra_a_styles:
+            Optional extra CSS styles for the <a> element.
+        escape_alt:
+            HTML-escape the alt text?
+        escape_text:
+            HTML-escape the main text?
+        hyperlink_together:
+            Hyperlink the image and text as one (rather than separately and
+            adjacent to each other)?
+    """
+    i_html = icon_html(icon=icon,
+                       url=None if hyperlink_together else url,
+                       alt=alt or text,
+                       extra_classes=extra_icon_classes,
+                       extra_styles=extra_icon_styles,
+                       escape_alt=escape_alt)
+    if escape_text:
+        text = html_escape(text)
+    if url:
+        a_components = [f'href="{url}"']
+        if extra_a_classes:
+            class_str = " ".join(extra_a_classes)
+            a_components.append(f'class="{class_str}"')
+        if extra_a_styles:
+            style_str = "; ".join(extra_a_styles)
+            a_components.append(f'style="{style_str}"')
+        a_options = " ".join(a_components)
+        if hyperlink_together:
+            return f'<a {a_options}>{i_html} {text}</a>'
+        else:
+            return f'{i_html} <a {a_options}>{text}</a>'
+    else:
+        return f'{i_html} {text}'
+
+
+def icons_text(icons: List[str],
+               text: str,
+               url: str = None,
+               alt: str = None,
+               extra_icon_classes: List[str] = None,
+               extra_icon_styles: List[str] = None,
+               extra_a_classes: List[str] = None,
+               extra_a_styles: List[str] = None,
+               escape_alt: bool = True,
+               escape_text: bool = True,
+               hyperlink_together: bool = False) -> str:
+    """
+    Multiple-icon version of :func:``icon_text``.
+    """
+    i_html = " ".join(
+        icon_html(icon=icon,
+                  url=None if hyperlink_together else url,
+                  alt=alt or text,
+                  extra_classes=extra_icon_classes,
+                  extra_styles=extra_icon_styles,
+                  escape_alt=escape_alt)
+        for icon in icons
+    )
+    if escape_text:
+        text = html_escape(text)
+    if url:
+        a_components = [f'href="{url}"']
+        if extra_a_classes:
+            class_str = " ".join(extra_a_classes)
+            a_components.append(f'class="{class_str}"')
+        if extra_a_styles:
+            style_str = "; ".join(extra_a_styles)
+            a_components.append(f'style="{style_str}"')
+        a_options = " ".join(a_components)
+        if hyperlink_together:
+            return f'<a {a_options}>{i_html} {text}</a>'
+        else:
+            return f'{i_html} <a {a_options}>{text}</a>'
+    else:
+        return f'{i_html} {text}'
+
+
+class Icons:
+    """
+    Constants for Bootstrap icons. See https://icons.getbootstrap.com/.
+    """
+    ACTIVITY = "activity"
+    APP_AUTHENTICATOR = "shield-shaded"
+    AUDIT_ITEM = "tag"
+    AUDIT_MENU = "clipboard"
+    AUDIT_OPTIONS = "clipboard-check"
+    AUDIT_REPORT = "clipboard-data"
+    BUSY = "hourglass-split"
+    COMPLETE = "check"
+    CTV = "body-text"
+    DELETE = "trash"
+    DELETE_MAJOR = "trash-fill"
+    DEVELOPER = "braces"  # braces, bug
+    DOWNLOAD = "download"
+    DUE = "alarm"
+    DUMP_BASIC = "file-spreadsheet"
+    DUMP_SQL = "server"
+    EDIT = "pencil"
+    EMAIL_CONFIGURE = "at"
+    EMAIL_SEND = "envelope"
+    EMAIL_VIEW = "envelope-open"
+    EXPORT_RECIPIENT = "share"
+    EXPORTED_TASK = "tag-fill"
+    EXPORTED_TASK_ENTRY_COLLECTION = "tags"
+    FILTER = "funnel"  # better than filter-circle
+    FORCE_FINALIZE = "bricks"
+    GITHUB = "github"
+    GOTO_PREDECESSOR = "arrow-left-square"
+    GOTO_SUCCESSOR = "arrow-right-square-fill"
+    GROUP_ADD = "plus-circle"
+    GROUP_ADMIN = "suit-diamond"
+    GROUP_EDIT = "box"
+    GROUPS = "boxes"  # change?
+    HOME = "house-fill"
+    HTML_ANONYMOUS = "file-richtext"
+    HTML_IDENTIFIABLE = "file-richtext-fill"
+    ID_DEFINITION_ADD = "plus-circle"  # suboptimal
+    ID_DEFINITIONS = "123"
+    INCOMPLETE = "x-circle"
+    INFO_EXTERNAL = "info-circle-fill"  # info-circle-fill? link? box-arrow-up-right?  # noqa
+    INFO_INTERNAL = "info-circle"
+    JSON = "file-text-fill"  # braces, file-text-fill
+    LOGIN = "box-arrow-in-right"
+    LOGOUT = "box-arrow-right"
+    MFA = "fingerprint"
+    MISSING = "x-octagon-fill"  # when an icon should have been supplied but wasn't!  # noqa
+    NAVIGATE_BACKWARD = "skip-start"
+    NAVIGATE_END = "skip-forward"  # better than skip-end
+    NAVIGATE_FORWARD = "skip-end"  # better than skip-forward, caret-right; "play" is also good but no mirror-image version.  # noqa
+    NAVIGATE_START = "skip-backward"  # better than skip-start
+    PASSWORD_OTHER = "key"
+    PASSWORD_OWN = "key-fill"
+    PATIENT = "person"
+    PATIENT_ADD = "person-plus"
+    PATIENT_EDIT = "person-circle"
+    PATIENTS = "people"
+    PDF_ANONYMOUS = "file-pdf"
+    PDF_IDENTIFIABLE = "file-pdf-fill"
+    REPORT_CONFIG = "bar-chart-line"
+    REPORT_DETAIL = "file-bar-graph"
+    REPORTS = "bar-chart-line-fill"
+    SETTINGS = "gear"
+    SMS = "chat-left-dots"
+    SPECIAL_NOTE = "pencil-square"
+    SUCCESS = "check-circle"
+    SUPERUSER = "suit-spade-fill"
+    TASK_SCHEDULE = "journal"
+    TASK_SCHEDULE_ADD = "journal-plus"
+    TASK_SCHEDULE_ITEM_ADD = "journal-code"  # imperfect, but we use journal-plus for "add schedule"  # noqa
+    TASK_SCHEDULE_ITEMS = "journal-text"
+    TASK_SCHEDULES = "journals"
+    TRACKERS = "graph-up"
+    UNKNOWN = "question-circle"
+    UNLOCK = "unlock"
+    UPLOAD = "upload"
+    USER_ADD = "person-plus-fill"  # there isn't a person-badge-plus
+    USER_INFO = "person-badge"
+    USER_MANAGEMENT = "person-badge-fill"
+    USER_PERMISSIONS = "person-check"
+    VIEW_TASKS = "display"
+    XML = "file-code-fill"  # diagram-3-fill
+    YOU = "heart-fill"
+    ZOOM_IN = "zoom-in"
+    ZOOM_OUT = "zoom-out"
+
+
 # =============================================================================
 # Pagination
 # =============================================================================
@@ -1147,6 +1557,16 @@ class SqlalchemyOrmQueryWrapper(object):
         return self.query.count()
 
 
+# DEFAULT_NAV_START = "&lt;&lt;"
+DEFAULT_NAV_START = icon_html(Icons.NAVIGATE_START, alt="Start")
+# DEFAULT_NAV_END = "&gt;&gt;"
+DEFAULT_NAV_END = icon_html(Icons.NAVIGATE_END, alt="End")
+# DEFAULT_NAV_BACKWARD = "&lt;"
+DEFAULT_NAV_BACKWARD = icon_html(Icons.NAVIGATE_BACKWARD, alt="Backward")
+# DEFAULT_NAV_FORWARD = '&gt;'
+DEFAULT_NAV_FORWARD = icon_html(Icons.NAVIGATE_FORWARD, alt="Forward")
+
+
 class CamcopsPage(Page):
     """
     Pagination class, for HTML views that display, for example,
@@ -1206,10 +1626,10 @@ class CamcopsPage(Page):
               url: str = None,
               show_if_single_page: bool = True,  # see below!
               separator: str = ' ',
-              symbol_first: str = '&lt;&lt;',
-              symbol_last: str = '&gt;&gt;',
-              symbol_previous: str = '&lt;',
-              symbol_next: str = '&gt;',
+              symbol_first: str = DEFAULT_NAV_START,
+              symbol_last: str = DEFAULT_NAV_END,
+              symbol_previous: str = DEFAULT_NAV_BACKWARD,
+              symbol_next: str = DEFAULT_NAV_FORWARD,
               link_attr: Dict[str, str] = None,
               curpage_attr: Dict[str, str] = None,
               dotdot_attr: Dict[str, str] = None,
@@ -1267,6 +1687,8 @@ class CamcopsPage(Page):
                  curpage_attr: Dict[str, str] = None,
                  dotdot_attr: Dict[str, str] = None):
         """
+        See equivalent in superclass.
+
         Fixes bugs (e.g. mutable default arguments) and nasties (e.g.
         enforcing ".." for the ellipsis) in the original.
         """
@@ -1291,11 +1713,11 @@ class CamcopsPage(Page):
         leftmost_page = (
             max(self.first_page, (self.page - radius))
             if self.first_page else None
-        )
+        )  # type: Optional[int]
         rightmost_page = (
             min(self.last_page, (self.page+radius))
             if self.last_page else None
-        )
+        )  # type: Optional[int]
         nav_items = {
             "first_page": None,
             "last_page": None,
