@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_session.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_session.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -49,9 +50,10 @@ from camcops_server.cc_modules.cc_constants import DateFormat
 from camcops_server.cc_modules.cc_pyramid import CookieKey
 from camcops_server.cc_modules.cc_sqla_coltypes import (
     IPAddressColType,
+    JsonColType,
     SessionTokenColType,
 )
-from camcops_server.cc_modules.cc_sqlalchemy import Base
+from camcops_server.cc_modules.cc_sqlalchemy import Base, MutableDict
 from camcops_server.cc_modules.cc_taskfilter import TaskFilter
 from camcops_server.cc_modules.cc_user import (
     User,
@@ -71,7 +73,8 @@ log = BraceStyleAdapter(logging.getLogger(__name__))
 DEBUG_CAMCOPS_SESSION_CREATION = False
 
 if DEBUG_CAMCOPS_SESSION_CREATION:
-    log.warning("cc_session: Debugging options enabled!")
+    log.warning("Debugging options enabled!")
+
 
 # =============================================================================
 # Constants
@@ -144,7 +147,11 @@ class CamcopsSession(Base):
         default=False,
         comment="This session is using the client API (not a human browsing)."
     )
-
+    form_state = Column(
+        "form_state", MutableDict.as_mutable(JsonColType),
+        comment=("Any state that needs to be saved temporarily during "
+                 "wizard-style form submission")
+    )
     user = relationship("User", lazy="joined", foreign_keys=[user_id])
     task_filter = relationship(
         "TaskFilter", foreign_keys=[task_filter_id],
@@ -284,12 +291,18 @@ class CamcopsSession(Base):
         if session_id and session_token:
             oldest_last_activity_allowed = \
                 cls.get_oldest_last_activity_allowed(req)
-            candidate = dbsession.query(cls).\
+            query = dbsession.query(cls).\
                 filter(cls.id == session_id).\
                 filter(cls.token == session_token).\
-                filter(cls.ip_address == ip_addr).\
-                filter(cls.last_activity_utc >= oldest_last_activity_allowed).\
-                first()  # type: Optional[CamcopsSession]
+                filter(cls.last_activity_utc >= oldest_last_activity_allowed)
+
+            if req.config.session_check_user_ip:
+                # Binding the session to the IP address can cause problems if
+                # the IP address changes before the session times out. A load
+                # balancer may cause this.
+                query = query.filter(cls.ip_address == ip_addr)
+
+            candidate = query.first()  # type: Optional[CamcopsSession]
             if DEBUG_CAMCOPS_SESSION_CREATION:
                 if candidate is None:
                     log.debug("Session not found in database")
