@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_sqla_coltypes.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,14 +21,14 @@ camcops_server/cc_modules/cc_sqla_coltypes.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
 **SQLAlchemy column types used by CamCOPS.**
 
 Note these built-in SQLAlchemy types
-(http://docs.sqlalchemy.org/en/latest/core/type_basics.html#generic-types):
+(https://docs.sqlalchemy.org/en/latest/core/type_basics.html#generic-types):
 
     =============== ===========================================================
     SQLAlchemy type Comment
@@ -99,8 +100,8 @@ Also notes:
 
 import json
 import logging
-from typing import (Any, Generator, List, Optional, Tuple, Type, TYPE_CHECKING,
-                    Union)
+from typing import (Any, Generator, List, Optional, Sequence,
+                    Tuple, Type, TYPE_CHECKING, Union)
 import uuid
 
 from cardinal_pythonlib.datetimefunc import (
@@ -127,6 +128,7 @@ from cardinal_pythonlib.sqlalchemy.sqlfunc import (
 from isodate.isoerror import ISO8601Error
 from pendulum import DateTime as Pendulum, Duration
 from pendulum.parsing.exceptions import ParserError
+import phonenumbers
 from semantic_version import Version
 from sqlalchemy import util
 from sqlalchemy.dialects import mysql
@@ -163,6 +165,7 @@ if TYPE_CHECKING:
 
 log = BraceStyleAdapter(logging.getLogger(__name__))
 
+
 # =============================================================================
 # Debugging options
 # =============================================================================
@@ -182,11 +185,11 @@ if any([DEBUG_DATETIME_AS_ISO_TEXT,
         DEBUG_STRING_LIST_COLTYPE]):
     log.warning("Debugging options enabled!")
 
+
 # =============================================================================
 # Constants
 # =============================================================================
 
-
 class RelationshipInfo(object):
     """
     Used as keys the ``info`` (user-defined) dictionary parameter to SQLAlchemy
@@ -209,6 +212,8 @@ AuditSourceColType = String(length=StringLengths.AUDIT_SOURCE_MAX_LEN)
 #     it's nonstandard and unnecessary, changed all BigIntUnsigned to
 #     BigInteger (2017-08-25).
 
+Base32ColType = String(length=StringLengths.BASE32_MAX_LEN)
+
 CharColType = String(length=1)
 CharsetColType = String(length=StringLengths.CHARSET_MAX_LEN)
 CurrencyColType = Unicode(length=StringLengths.CURRENCY_MAX_LEN)
@@ -267,6 +272,7 @@ LongBlob = LargeBinary().with_variant(mysql.LONGBLOB, "mysql")
 LongText = UnicodeText().with_variant(mysql.LONGTEXT, "mysql")
 # LongText = UnicodeText(length=LONGBLOB_LONGTEXT_MAX_LEN)  # doesn't translate to SQL Server  # noqa
 
+MfaMethodColType = String(length=StringLengths.MFA_METHOD_MAX_LEN)
 MimeTypeColType = String(length=StringLengths.MIMETYPE_MAX_LEN)
 
 PatientNameColType = Unicode(length=StringLengths.PATIENT_NAME_MAX_LEN)
@@ -352,7 +358,7 @@ def isotzdatetime_to_utcdatetime_mysql(
     Note the use of "%i" for minutes.
 
     Things after ``func.`` get passed to the database engine as literal SQL
-    functions; http://docs.sqlalchemy.org/en/latest/core/tutorial.html
+    functions; https://docs.sqlalchemy.org/en/latest/core/tutorial.html
     """  # noqa
     x = fetch_processed_single_clause(element, compiler)
 
@@ -369,7 +375,7 @@ def isotzdatetime_to_utcdatetime_mysql(
         f"CONVERT_TZ({the_date_time}, {old_timezone}, {_UTC_TZ_LITERAL})"
     )
 
-    # log.warning(result_utc)
+    # log.debug(result_utc)
     return result_utc
 
 
@@ -383,7 +389,7 @@ def isotzdatetime_to_utcdatetime_sqlite(
 
     - https://sqlite.org/lang_corefunc.html#substr
     - https://sqlite.org/lang_datefunc.html
-    - http://www.sqlite.org/lang_expr.html
+    - https://www.sqlite.org/lang_expr.html
 
     Get an SQL expression for the timezone adjustment in hours.
     Note that if a time is 12:00+01:00, that means e.g. midday BST, which
@@ -423,7 +429,7 @@ def isotzdatetime_to_utcdatetime_sqlite(
     x = fetch_processed_single_clause(element, compiler)
     fmt = compiler.process(text(_SQLITE_DATETIME_FMT_FOR_PYTHON))
     result = f"(STRFTIME({fmt}, {x}) || '000')"
-    # log.warning(result)
+    # log.debug(result)
     return result
 
 
@@ -543,7 +549,7 @@ def isotzdatetime_to_utcdatetime_sqlserver(
     )
     result_utc = f"CAST({date_time_offset_with_utc_tz} AS DATETIME2)"
 
-    # log.warning(result_utc)
+    # log.debug(result_utc)
     return result_utc
 
 
@@ -591,7 +597,7 @@ def unknown_field_to_utcdatetime_mysql(
     x = fetch_processed_single_clause(element, compiler)
     converted = isotzdatetime_to_utcdatetime_mysql(element, compiler, **kw)
     result = f"IF(LENGTH({x}) = {_MYSQL_DATETIME_LEN}, {x}, {converted})"
-    # log.warning(result)
+    # log.debug(result)
     return result
 
 
@@ -606,7 +612,7 @@ def unknown_field_to_utcdatetime_sqlite(
     x = fetch_processed_single_clause(element, compiler)
     fmt = compiler.process(text(_SQLITE_DATETIME_FMT_FOR_PYTHON))
     result = f"STRFTIME({fmt}, {x})"
-    # log.warning(result)
+    # log.debug(result)
     return result
 
 
@@ -638,7 +644,7 @@ def unknown_field_to_utcdatetime_sqlserver(
         f"ELSE {converted} "
         f"END"
     )
-    # log.warning(result)
+    # log.debug(result)
     return result
 
 
@@ -698,7 +704,7 @@ class PendulumDateTimeAsIsoTextColType(TypeDecorator):
         """
         retval = self.pendulum_to_isostring(value)
         if DEBUG_DATETIME_AS_ISO_TEXT:
-            log.warning(
+            log.debug(
                 "{}.process_bind_param("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -711,7 +717,7 @@ class PendulumDateTimeAsIsoTextColType(TypeDecorator):
         """
         retval = self.pendulum_to_isostring(value)
         if DEBUG_DATETIME_AS_ISO_TEXT:
-            log.warning(
+            log.debug(
                 "{}.process_literal_param("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -724,7 +730,7 @@ class PendulumDateTimeAsIsoTextColType(TypeDecorator):
         """
         retval = self.isostring_to_pendulum(value)
         if DEBUG_DATETIME_AS_ISO_TEXT:
-            log.warning(
+            log.debug(
                 "{}.process_result_value("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -770,10 +776,10 @@ class PendulumDateTimeAsIsoTextColType(TypeDecorator):
                 # DATETIME, then we assume it is already in UTC.
                 processed_other = unknown_field_to_utcdatetime(other)
             if DEBUG_DATETIME_AS_ISO_TEXT:
-                log.warning("operate(self={!r}, op={!r}, other={!r})",
-                            self, op, other)
-                log.warning("self.expr = {!r}", self.expr)
-                log.warning("processed_other = {!r}", processed_other)
+                log.debug("operate(self={!r}, op={!r}, other={!r})",
+                          self, op, other)
+                log.debug("self.expr = {!r}", self.expr)
+                log.debug("processed_other = {!r}", processed_other)
                 # traceback.print_stack()
             return op(isotzdatetime_to_utcdatetime(self.expr),
                       processed_other)
@@ -836,7 +842,7 @@ class PendulumDurationAsIsoTextColType(TypeDecorator):
         """
         retval = self.pendulum_duration_to_isostring(value)
         if DEBUG_DURATION_AS_ISO_TEXT:
-            log.warning(
+            log.debug(
                 "{}.process_bind_param("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -849,7 +855,7 @@ class PendulumDurationAsIsoTextColType(TypeDecorator):
         """
         retval = self.pendulum_duration_to_isostring(value)
         if DEBUG_DURATION_AS_ISO_TEXT:
-            log.warning(
+            log.debug(
                 "{}.process_literal_param("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -862,7 +868,7 @@ class PendulumDurationAsIsoTextColType(TypeDecorator):
         """
         retval = self.isostring_to_pendulum_duration(value)
         if DEBUG_DURATION_AS_ISO_TEXT:
-            log.warning(
+            log.debug(
                 "{}.process_result_value("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -899,7 +905,7 @@ class SemanticVersionColType(TypeDecorator):
         """
         retval = str(value) if value is not None else None
         if DEBUG_SEMANTIC_VERSION:
-            log.warning(
+            log.debug(
                 "{}.process_bind_param("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -912,7 +918,7 @@ class SemanticVersionColType(TypeDecorator):
         """
         retval = str(value) if value is not None else None
         if DEBUG_SEMANTIC_VERSION:
-            log.warning(
+            log.debug(
                 "{}.process_literal_param("
                 "self={!r}, value={!r}, dialect={!r}) -> !r",
                 self._coltype_name, self, value, dialect, retval)
@@ -931,7 +937,7 @@ class SemanticVersionColType(TypeDecorator):
             # ordered Version out:
             retval = make_version(value)
         if DEBUG_SEMANTIC_VERSION:
-            log.warning(
+            log.debug(
                 "{}.process_result_value("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -1035,7 +1041,7 @@ class IdNumReferenceListColType(TypeDecorator):
         """
         retval = self._idnumdef_list_to_dbstr(value)
         if DEBUG_IDNUMDEF_LIST:
-            log.warning(
+            log.debug(
                 "{}.process_bind_param("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -1048,7 +1054,7 @@ class IdNumReferenceListColType(TypeDecorator):
         """
         retval = self._idnumdef_list_to_dbstr(value)
         if DEBUG_IDNUMDEF_LIST:
-            log.warning(
+            log.debug(
                 "{}.process_literal_param("
                 "self={!r}, value={!r}, dialect={!r}) -> !r",
                 self._coltype_name, self, value, dialect, retval)
@@ -1061,7 +1067,7 @@ class IdNumReferenceListColType(TypeDecorator):
         """
         retval = self._dbstr_to_idnumdef_list(value)
         if DEBUG_IDNUMDEF_LIST:
-            log.warning(
+            log.debug(
                 "{}.process_result_value("
                 "self={!r}, value={!r}, dialect={!r}) -> {!r}",
                 self._coltype_name, self, value, dialect, retval)
@@ -1126,6 +1132,33 @@ class JsonColType(TypeDecorator):
         return json.loads(value)
 
 
+# =============================================================================
+# Phone number column type
+# =============================================================================
+
+class PhoneNumberColType(TypeDecorator):
+    impl = Unicode(length=StringLengths.PHONE_NUMBER_MAX_LEN)
+
+    @property
+    def python_type(self) -> type:
+        return str
+
+    def process_bind_param(self, value: Any,
+                           dialect: Dialect) -> Optional[str]:
+        if value is None:
+            return None
+
+        return phonenumbers.format_number(value,
+                                          phonenumbers.PhoneNumberFormat.E164)
+
+    def process_result_value(self, value: str, dialect: Dialect) -> Any:
+        if not value:
+            return None
+
+        # Should be stored as E164 so no need to pass a region
+        return phonenumbers.parse(value, None)
+
+
 # =============================================================================
 # PermittedValueChecker: used by CamcopsColumn
 # =============================================================================
@@ -1139,7 +1172,7 @@ class PermittedValueChecker(object):
                  not_null: bool = False,
                  minimum: Union[int, float] = None,
                  maximum: Union[int, float] = None,
-                 permitted_values: List[Any] = None) -> None:
+                 permitted_values: Sequence[Any] = None) -> None:
         """
         Args:
             not_null: must the value not be NULL?
@@ -1191,20 +1224,26 @@ class PermittedValueChecker(object):
     def __repr__(self):
         return auto_repr(self)
 
+    def permitted_values_inc_minmax(self) -> Tuple:
+        """
+        Returns permitted values, either specified directly or via a
+        minimum/maximum.
+        """
+        if self.permitted_values:
+            return tuple(self.permitted_values)
+        # Take a punt that integer minima/maxima mean that only integers are
+        # permitted...
+        if isinstance(self.minimum, int) and isinstance(self.maximum, int):
+            return tuple(range(self.minimum, self.maximum + 1))
+        return ()
+
     def permitted_values_csv(self) -> str:
         """
         Returns a CSV representation of the permitted values.
 
         Primarily used for CRIS data dictionaries.
         """
-        if self.permitted_values:
-            return ",".join(str(x) for x in self.permitted_values)
-        # Take a punt that integer minima/maxima mean that only integers are
-        # permitted...
-        if isinstance(self.minimum, int) and isinstance(self.maximum, int):
-            return ",".join(
-                str(x) for x in range(self.minimum, self.maximum + 1))
-        return ""
+        return ",".join(str(x) for x in self.permitted_values_inc_minmax())
 
 
 # Specific instances, to reduce object duplication and magic numbers:
@@ -1238,6 +1277,10 @@ ONE_TO_NINE_CHECKER = PermittedValueChecker(minimum=1, maximum=9)
 # CamcopsColumn: provides extra functions over Column.
 # =============================================================================
 
+# Column attributes:
+COLATTR_PERMITTED_VALUE_CHECKER = "permitted_value_checker"
+
+
 # noinspection PyAbstractClass
 class CamcopsColumn(Column):
     """
@@ -1313,7 +1356,7 @@ class CamcopsColumn(Column):
         kwargs['identifies_patient'] = self.identifies_patient
         kwargs['is_blob_id_field'] = self.is_blob_id_field
         kwargs['blob_relationship_attr_name'] = self.blob_relationship_attr_name  # noqa
-        kwargs['permitted_value_checker'] = self.permitted_value_checker
+        kwargs[COLATTR_PERMITTED_VALUE_CHECKER] = self.permitted_value_checker
         # noinspection PyTypeChecker
         return self.__class__(*args, **kwargs)
 
@@ -1326,7 +1369,7 @@ class CamcopsColumn(Column):
             kvp("identifies_patient"),
             kvp("is_blob_id_field"),
             kvp("blob_relationship_attr_name"),
-            kvp("permitted_value_checker"),
+            kvp(COLATTR_PERMITTED_VALUE_CHECKER),
             f"super()={super().__repr__()}",
         ]
         return f"CamcopsColumn({', '.join(elements)})"
@@ -1562,14 +1605,12 @@ class BoolColumn(CamcopsColumn):
             kwargs['type_'] = Boolean(name=constraint_name_conv)
             # The "name" parameter to Boolean() specifies the  name of the
             # (0, 1) constraint.
-        kwargs['permitted_value_checker'] = BIT_CHECKER
+        kwargs[COLATTR_PERMITTED_VALUE_CHECKER] = BIT_CHECKER
         super().__init__(*args, **kwargs)
         if (not self.constraint_name and
                 len(self.name) >= LONG_COLUMN_NAME_WARNING_LIMIT):
-            log.warning(
-                "BoolColumn with long column name and no constraint name: "
-                "{!r}", self.name
-            )
+            log.warning("BoolColumn with long column name and no constraint "
+                        "name: {!r}", self.name)
 
     def __repr__(self) -> str:
         def kvp(attrname: str) -> str:
