@@ -5,7 +5,8 @@ camcops_server/cc_modules/cc_taskschedule.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/cc_taskschedule.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -28,22 +29,25 @@ camcops_server/cc_modules/cc_taskschedule.py
 
 import logging
 from typing import List, Iterable, Optional, Tuple, TYPE_CHECKING
-from urllib.parse import quote, urlencode
+from urllib.parse import urlencode, urlunsplit
 
+from cardinal_pythonlib.uriconst import UriSchemes
 from pendulum import DateTime as Pendulum, Duration
 
 from sqlalchemy import cast, Numeric
 from sqlalchemy.orm import relationship
 from sqlalchemy.sql.functions import func
 from sqlalchemy.sql.schema import Column, ForeignKey
-from sqlalchemy.sql.sqltypes import Integer, UnicodeText
+from sqlalchemy.sql.sqltypes import BigInteger, Integer, UnicodeText
 
+from camcops_server.cc_modules.cc_email import Email
 from camcops_server.cc_modules.cc_formatter import SafeFormatter
 from camcops_server.cc_modules.cc_group import Group
 from camcops_server.cc_modules.cc_pyramid import Routes
 from camcops_server.cc_modules.cc_simpleobjects import IdNumReference
 from camcops_server.cc_modules.cc_sqlalchemy import Base
 from camcops_server.cc_modules.cc_sqla_coltypes import (
+    EmailAddressColType,
     JsonColType,
     PendulumDateTimeAsIsoTextColType,
     PendulumDurationAsIsoTextColType,
@@ -87,6 +91,41 @@ class ScheduledTaskInfo(object):
         self.start_datetime = start_datetime
         self.end_datetime = end_datetime
 
+    @property
+    def due_now(self) -> bool:
+        """
+        Are we in the range [start_datetime, end_datetime)?
+        """
+        if not self.start_datetime or not self.end_datetime:
+            return False
+        return self.start_datetime <= Pendulum.now() < self.end_datetime
+
+    @property
+    def is_complete(self) -> bool:
+        """
+        Returns whether its associated task is complete..
+        """
+        if not self.task:
+            return False
+        return self.task.is_complete()
+
+    @property
+    def is_identifiable_and_incomplete(self) -> bool:
+        """
+        If this is an anonymous task, returns ``False``.
+        If this is an identifiable task, returns ``not is_complete``.
+        """
+        if self.is_anonymous:
+            return False
+        return not self.is_complete
+
+    @property
+    def due_now_identifiable_and_incomplete(self) -> bool:
+        """
+        Is this task currently due, identifiable, and incomplete?
+        """
+        return self.due_now and self.is_identifiable_and_incomplete
+
 
 # =============================================================================
 # PatientTaskSchedule
@@ -101,11 +140,13 @@ class PatientTaskSchedule(Base):
     id = Column("id", Integer, primary_key=True, autoincrement=True)
     patient_pk = Column(
         "patient_pk", Integer,
-        ForeignKey("patient._pk", ondelete="CASCADE")
+        ForeignKey("patient._pk"),
+        nullable=False,
     )
     schedule_id = Column(
         "schedule_id", Integer,
-        ForeignKey("_task_schedule.id", ondelete="CASCADE")
+        ForeignKey("_task_schedule.id"),
+        nullable=False,
     )
     start_datetime = Column(
         "start_datetime", PendulumDateTimeAsIsoTextColType,
@@ -119,11 +160,26 @@ class PatientTaskSchedule(Base):
         comment="Task-specific settings for this patient"
     )
 
-    patient = relationship("Patient", back_populates="task_schedules")
-    task_schedule = relationship("TaskSchedule", back_populates="patients")
+    patient = relationship(
+        "Patient",
+        back_populates="task_schedules"
+    )
+    task_schedule = relationship(
+        "TaskSchedule",
+        back_populates="patient_task_schedules"
+    )
+
+    emails = relationship(
+        "PatientTaskScheduleEmail",
+        back_populates="patient_task_schedule",
+        cascade="all, delete"
+    )
 
     def get_list_of_scheduled_tasks(self, req: "CamcopsRequest") \
             -> List[ScheduledTaskInfo]:
+        """
+        Tasks scheduled for this patient.
+        """
 
         task_list = []
 
@@ -204,24 +260,48 @@ class PatientTaskSchedule(Base):
 
         return None
 
-    def mailto_url(self, req: "CamcopsRequest") -> str:
+    def email_body(self, req: "CamcopsRequest") -> str:
+        """
+        Body content (HTML) for an e-mail to the patient -- the schedule's
+        template, populated with patient-specific information.
+        """
         template_dict = dict(
             access_key=self.patient.uuid_as_proquint,
-            server_url=req.route_url(Routes.CLIENT_API)
+            android_launch_url=self.launch_url(req, UriSchemes.HTTP),
+            ios_launch_url=self.launch_url(req, "camcops"),
+            forename=self.patient.forename,
+            server_url=req.route_url(Routes.CLIENT_API),
+            surname=self.patient.surname,
         )
 
         formatter = TaskScheduleEmailTemplateFormatter()
-        email_body = formatter.format(self.task_schedule.email_template,
-                                      **template_dict)
+        return formatter.format(self.task_schedule.email_template,
+                                **template_dict)
 
-        mailto_params = urlencode({
-            "subject": self.task_schedule.email_subject,
-            "body": email_body,
-        }, quote_via=quote)
+    def launch_url(self, req: "CamcopsRequest", scheme: str) -> str:
+        # Matches intent-filter in AndroidManifest.xml
+        # And CFBundleURLSchemes in Info.plist
 
-        mailto_url = f"mailto:{self.patient.email}?{mailto_params}"
+        # iOS doesn't care about these:
+        netloc = "camcops.org"
+        path = "/register/"
+        fragment = ""
 
-        return mailto_url
+        query_dict = {
+            "default_single_user_mode": "true",
+            "default_server_location": req.route_url(Routes.CLIENT_API),
+            "default_access_key": self.patient.uuid_as_proquint,
+        }
+        query = urlencode(query_dict)
+
+        return urlunsplit((scheme, netloc, path, query, fragment))
+
+    @property
+    def email_sent(self) -> bool:
+        """
+        Has an e-mail been sent to the patient for this schedule?
+        """
+        return any(e.email.sent for e in self.emails)
 
 
 def task_schedule_item_sort_order() -> Tuple["Cast", "Cast"]:
@@ -234,6 +314,14 @@ def task_schedule_item_sort_order() -> Tuple["Cast", "Cast"]:
     to get them in the order we want.
 
     This will fail if durations ever get stored any other way.
+
+    Note that MySQL does not permit "CAST(... AS DOUBLE)" or "CAST(... AS
+    FLOAT)"; you need to use NUMERIC or DECIMAL. However, this raises a warning
+    when running self-tests under SQLite: "SAWarning: Dialect sqlite+pysqlite
+    does *not* support Decimal objects natively, and SQLAlchemy must convert
+    from floating point - rounding errors and other issues may occur. Please
+    consider storing Decimal numbers as strings or integers on this platform
+    for lossless storage."
     """
     due_from_order = cast(func.substr(TaskScheduleItem.due_from, 7),
                           Numeric())
@@ -243,6 +331,37 @@ def task_schedule_item_sort_order() -> Tuple["Cast", "Cast"]:
     return due_from_order, due_by_order
 
 
+# =============================================================================
+# Emails sent to patient
+# =============================================================================
+
+class PatientTaskScheduleEmail(Base):
+    """
+    Represents an email send to a patient for a particular task schedule.
+    """
+    __tablename__ = "_patient_task_schedule_email"
+
+    id = Column(
+        "id", Integer, primary_key=True, autoincrement=True,
+        comment="Arbitrary primary key"
+    )
+    patient_task_schedule_id = Column(
+        "patient_task_schedule_id", Integer, ForeignKey(PatientTaskSchedule.id),
+        nullable=False,
+        comment=(f"FK to {PatientTaskSchedule.__tablename__}."
+                 f"{PatientTaskSchedule.id.name}")
+    )
+    email_id = Column(
+        "email_id", BigInteger, ForeignKey(Email.id),
+        nullable=False,
+        comment=f"FK to {Email.__tablename__}.{Email.id.name}"
+    )
+
+    patient_task_schedule = relationship(PatientTaskSchedule,
+                                         back_populates="emails")
+    email = relationship(Email, cascade="all, delete")
+
+
 # =============================================================================
 # Task schedule
 # =============================================================================
@@ -273,18 +392,35 @@ class TaskSchedule(Base):
     email_template = Column("email_template", UnicodeText,
                             comment="email template", nullable=False,
                             default="")
+    email_from = Column("email_from", EmailAddressColType,
+                        comment="Sender's e-mail address")
+    email_cc = Column(
+        "email_cc", UnicodeText,
+        comment="Send a carbon copy of the email to these addresses"
+    )
+    email_bcc = Column(
+        "email_bcc", UnicodeText,
+        comment="Send a blind carbon copy of the email to these addresses"
+    )
 
     items = relationship(
         "TaskScheduleItem",
-        order_by=task_schedule_item_sort_order
+        order_by=task_schedule_item_sort_order,
+        cascade="all, delete"
     )  # type: Iterable[TaskScheduleItem]
 
     group = relationship(Group)
 
-    patients = relationship("PatientTaskSchedule",
-                            back_populates="task_schedule")
+    patient_task_schedules = relationship(
+        "PatientTaskSchedule",
+        back_populates="task_schedule",
+        cascade="all, delete"
+    )
 
     def user_may_edit(self, req: "CamcopsRequest") -> bool:
+        """
+        May the current user edit this schedule?
+        """
         return req.user.may_administer_group(self.group_id)
 
 
@@ -327,12 +463,19 @@ class TaskScheduleItem(Base):
 
     @property
     def task_shortname(self) -> str:
+        """
+        Short name of the task being scheduled.
+        """
         task_class_lookup = tablename_to_task_class_dict()
 
         return task_class_lookup[self.task_table_name].shortname
 
     @property
     def due_within(self) -> Optional[Duration]:
+        """
+        Returns the "due within" property, e.g. "due within 7 days (of being
+        scheduled to start)". This is calculated from due_from and due_by.
+        """
         if self.due_by is None:
             # Should not be possible if created through the form
             return None
@@ -343,6 +486,9 @@ class TaskScheduleItem(Base):
         return self.due_by - self.due_from
 
     def description(self, req: "CamcopsRequest") -> str:
+        """
+        Description of this schedule item -- which task, due when.
+        """
         _ = req.gettext
 
         if self.due_from is None:
@@ -358,5 +504,15 @@ class TaskScheduleItem(Base):
 
 
 class TaskScheduleEmailTemplateFormatter(SafeFormatter):
+    """
+    Safe template formatter for task schedule e-mails.
+    """
     def __init__(self):
-        super().__init__(["access_key", "server_url"])
+        super().__init__([
+            "access_key",
+            "android_launch_url",
+            "forename",
+            "ios_launch_url",
+            "server_url",
+            "surname",
+        ])
