@@ -5,7 +5,8 @@ camcops_server/cc_modules/tests/cc_forms_tests.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,13 +21,14 @@ camcops_server/cc_modules/tests/cc_forms_tests.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
 """
 
 import json
+import logging
 from pprint import pformat
 from typing import Any, Dict
 from unittest import mock, TestCase
@@ -34,6 +36,7 @@ from unittest import mock, TestCase
 # noinspection PyProtectedMember
 from colander import Invalid, null, Schema
 from pendulum import Duration
+import phonenumbers
 
 from camcops_server.cc_modules.cc_baseconstants import TEMPLATE_DIR
 from camcops_server.cc_modules.cc_forms import (
@@ -41,21 +44,33 @@ from camcops_server.cc_modules.cc_forms import (
     DurationWidget,
     GroupIpUseWidget,
     IpUseType,
+    MfaSecretWidget,
     JsonType,
     JsonWidget,
     LoginSchema,
+    PhoneNumberType,
     TaskScheduleItemSchema,
     TaskScheduleNode,
     TaskScheduleSchema,
+    TaskScheduleSelector,
 )
 from camcops_server.cc_modules.cc_ipuse import IpContexts
 from camcops_server.cc_modules.cc_pyramid import ViewParam
 from camcops_server.cc_modules.cc_taskschedule import TaskSchedule
 from camcops_server.cc_modules.cc_unittest import (
+    BasicDatabaseTestCase,
     DemoDatabaseTestCase,
     DemoRequestTestCase,
 )
 
+TEST_PHONE_NUMBER = "+{ctry}{tel}".format(
+    ctry=phonenumbers.PhoneMetadata.metadata_for_region("GB").country_code,
+    tel=phonenumbers.PhoneMetadata.metadata_for_region(
+        "GB").personal_number.example_number
+)  # see webview_tests.py
+
+log = logging.getLogger(__name__)
+
 
 # =============================================================================
 # Unit tests
@@ -97,14 +112,16 @@ class LoginSchemaTests(SchemaTestCase):
 class TaskScheduleSchemaTests(DemoDatabaseTestCase):
     def test_invalid_for_bad_template_placeholder(self) -> None:
         schema = TaskScheduleSchema().bind(request=self.req)
-        appstruct = {
+        cstruct = {
             ViewParam.NAME: "test",
-            ViewParam.GROUP_ID: "1",
+            ViewParam.GROUP_ID: str(self.group.id),
+            ViewParam.EMAIL_FROM: null,
+            ViewParam.EMAIL_CC: null,
+            ViewParam.EMAIL_BCC: null,
             ViewParam.EMAIL_SUBJECT: "Subject",
             ViewParam.EMAIL_TEMPLATE: "{bad_key}",
         }
 
-        cstruct = schema.serialize(appstruct)
         with self.assertRaises(Invalid) as cm:
             schema.deserialize(cstruct)
 
@@ -113,14 +130,16 @@ class TaskScheduleSchemaTests(DemoDatabaseTestCase):
 
     def test_invalid_for_mismatched_braces(self) -> None:
         schema = TaskScheduleSchema().bind(request=self.req)
-        appstruct = {
+        cstruct = {
             ViewParam.NAME: "test",
-            ViewParam.GROUP_ID: "1",
+            ViewParam.GROUP_ID: str(self.group.id),
+            ViewParam.EMAIL_FROM: null,
+            ViewParam.EMAIL_CC: null,
+            ViewParam.EMAIL_BCC: null,
             ViewParam.EMAIL_SUBJECT: "Subject",
-            ViewParam.EMAIL_TEMPLATE: "{server_url",
+            ViewParam.EMAIL_TEMPLATE: "{server_url",  # deliberately missing }
         }
 
-        cstruct = schema.serialize(appstruct)
         with self.assertRaises(Invalid) as cm:
             schema.deserialize(cstruct)
 
@@ -224,7 +243,7 @@ class TaskScheduleItemSchemaTests(SchemaTestCase):
                       cm.exception.messages()[0])
 
 
-class TaskScheduleItemSchemaIpTests(DemoDatabaseTestCase):
+class TaskScheduleItemSchemaIpTests(BasicDatabaseTestCase):
     def setUp(self) -> None:
         super().setUp()
 
@@ -233,10 +252,6 @@ class TaskScheduleItemSchemaIpTests(DemoDatabaseTestCase):
         self.dbsession.add(self.schedule)
         self.dbsession.commit()
 
-    def create_tasks(self) -> None:
-        # Speed things up a bit
-        pass
-
     def test_invalid_for_commercial_mismatch(self) -> None:
         self.group.ip_use.commercial = True
         self.dbsession.add(self.group)
@@ -705,6 +720,35 @@ class TaskScheduleNodeTests(TestCase):
             self.assertEqual(cm.exception.value, "[{}]")
 
 
+class TaskScheduleSelectorTests(BasicDatabaseTestCase):
+    def test_displays_only_users_schedules(self) -> None:
+        user = self.create_user(username="regular_user")
+        my_group = self.create_group("mygroup")
+        not_my_group = self.create_group("notmygroup")
+        self.dbsession.flush()
+
+        self.create_membership(user, my_group, may_manage_patients=True)
+
+        my_schedule = TaskSchedule()
+        my_schedule.group_id = my_group.id
+        my_schedule.name = "My group's schedule"
+        self.dbsession.add(my_schedule)
+
+        not_my_schedule = TaskSchedule()
+        not_my_schedule.group_id = not_my_group.id
+        not_my_schedule.name = "Not my group's schedule"
+        self.dbsession.add(not_my_schedule)
+        self.dbsession.commit()
+
+        self.req._debugging_user = user
+
+        selector = TaskScheduleSelector().bind(request=self.req)
+        self.assertIn((my_schedule.id, my_schedule.name),
+                      selector.widget.values)
+        self.assertNotIn((not_my_schedule.id, not_my_schedule.name),
+                         selector.widget.values)
+
+
 class GroupIpUseWidgetTests(TestCase):
     def setUp(self) -> None:
         super().setUp()
@@ -870,3 +914,129 @@ class IpUseTypeTests(TestCase):
         self.assertTrue(ip_use.commercial)
         self.assertFalse(ip_use.educational)
         self.assertTrue(ip_use.research)
+
+
+class MfaSecretWidgetTests(TestCase):
+    def setUp(self) -> None:
+        super().setUp()
+        self.request = mock.Mock(gettext=lambda t: t,
+                                 user=mock.Mock(username="test"))
+        self.mfa_secret = "HVIHV7TUFQPV7KAIJE2GSJTLTEAQIQSJ"
+
+    def test_serialize_renders_template_with_values(self) -> None:
+        widget = MfaSecretWidget(self.request)
+
+        field = mock.Mock()
+        field.renderer = mock.Mock()
+
+        cstruct = self.mfa_secret
+        widget.serialize(field, cstruct, readonly=False)
+
+        args, kwargs = field.renderer.call_args
+
+        self.assertEqual(args[0], f"{TEMPLATE_DIR}/deform/mfa_secret.pt")
+        self.assertFalse(kwargs["readonly"])
+
+        self.assertIn("<svg", kwargs["qr_code"])
+
+    def test_serialize_renders_readonly_template(self) -> None:
+        widget = MfaSecretWidget(self.request)
+
+        field = mock.Mock()
+        field.renderer = mock.Mock()
+
+        cstruct = self.mfa_secret
+        widget.serialize(field, cstruct, readonly=True)
+
+        args, kwargs = field.renderer.call_args
+
+        self.assertEqual(args[0],
+                         f"{TEMPLATE_DIR}/deform/readonly/mfa_secret.pt")
+        self.assertTrue(kwargs["readonly"])
+
+    def test_serialize_readonly_widget_renders_readonly_template(self) -> None:
+        widget = MfaSecretWidget(self.request, readonly=True)
+
+        field = mock.Mock()
+        field.renderer = mock.Mock()
+
+        cstruct = self.mfa_secret
+        widget.serialize(field, cstruct)
+
+        args, kwargs = field.renderer.call_args
+
+        self.assertEqual(args[0],
+                         f"{TEMPLATE_DIR}/deform/readonly/mfa_secret.pt")
+
+
+class PhoneNumberTypeTestCase(TestCase):
+    def setUp(self) -> None:
+        super().setUp()
+
+        self.request = mock.Mock()
+        self.phone_type = PhoneNumberType(self.request, allow_empty=True)
+        self.node = mock.Mock()
+
+
+class PhoneNumberTypeDeserializeTests(PhoneNumberTypeTestCase):
+    def test_returns_null_for_null_cstruct(self) -> None:
+        # For allow_empty=True:
+        phone_number = self.phone_type.deserialize(self.node, null)
+        self.assertIs(phone_number, null)
+
+    def test_raises_for_unparsable_number(self) -> None:
+        with self.assertRaises(Invalid) as cm:
+            self.phone_type.deserialize(self.node, "abc")
+
+            self.assertIn(
+                "Invalid phone number",
+                cm.exception.messages()[0]
+            )
+
+    def test_raises_for_invalid_parsable_number(self) -> None:
+        with self.assertRaises(Invalid) as cm:
+            self.phone_type.deserialize(self.node, "+4411349600")
+
+            self.assertIn(
+                "Invalid phone number",
+                cm.exception.messages()[0]
+            )
+
+    def test_returns_valid_phone_number(self) -> None:
+        phone_number = self.phone_type.deserialize(
+            self.node, TEST_PHONE_NUMBER)
+
+        self.assertIsInstance(phone_number, phonenumbers.PhoneNumber)
+
+        self.assertEqual(phonenumbers.format_number(
+            phone_number,
+            phonenumbers.PhoneNumberFormat.E164
+        ), TEST_PHONE_NUMBER)
+
+
+class PhoneNumberTypeSerializeTests(PhoneNumberTypeTestCase):
+    def test_returns_null_for_appstruct_none(self) -> None:
+        self.assertIs(self.phone_type.serialize(self.node, None), null)
+
+    def test_returns_number_formatted_e164(self) -> None:
+        phone_number = phonenumbers.parse(TEST_PHONE_NUMBER)
+
+        self.assertEqual(self.phone_type.serialize(self.node, phone_number),
+                         TEST_PHONE_NUMBER)
+
+
+class PhoneNumberTypeMandatoryTestCase(TestCase):
+    def setUp(self) -> None:
+        super().setUp()
+
+        self.request = mock.Mock()
+        self.phone_type = PhoneNumberType(self.request, allow_empty=False)
+        self.node = mock.Mock()
+
+
+class PhoneNumberTypeMandatoryDeserializeTests(
+        PhoneNumberTypeMandatoryTestCase):
+    def test_raises_for_appstruct_none(self) -> None:
+        # For allow_empty=False:
+        with self.assertRaises(Invalid):
+            self.phone_type.deserialize(self.node, null)
