@@ -5,7 +5,8 @@ camcops_server/cc_modules/tests/client_api_tests.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/cc_modules/tests/client_api_tests.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -29,13 +30,17 @@ camcops_server/cc_modules/tests/client_api_tests.py
 import json
 # from pprint import pformat
 import string
+from typing import Dict
 
+from cardinal_pythonlib.classes import class_attribute_names
 from cardinal_pythonlib.convert import (
     base64_64format_encode,
     hex_xformat_encode,
 )
+from cardinal_pythonlib.nhs import generate_random_nhs_number
 from cardinal_pythonlib.sql.literals import sql_quote_string
 from cardinal_pythonlib.text import escape_newlines, unescape_newlines
+from pyramid.response import Response
 
 from camcops_server.cc_modules.cc_client_api_core import (
     fail_server_error,
@@ -53,8 +58,10 @@ from camcops_server.cc_modules.cc_proquint import (
     uuid_from_proquint,
 )
 from camcops_server.cc_modules.cc_taskindex import update_indexes_and_push_exports  # noqa
-from camcops_server.cc_modules.cc_testhelpers import class_attribute_names
-from camcops_server.cc_modules.cc_unittest import DemoDatabaseTestCase
+from camcops_server.cc_modules.cc_unittest import (
+    BasicDatabaseTestCase,
+    DemoDatabaseTestCase
+)
 from camcops_server.cc_modules.cc_user import User
 from camcops_server.cc_modules.cc_version import (
     MINIMUM_TABLET_VERSION,
@@ -66,14 +73,44 @@ from camcops_server.cc_modules.client_api import (
     client_api,
     DEVICE_STORED_VAR_TABLENAME_DEFUNCT,
     FAILURE_CODE,
-    get_reply_dict_from_response,
     make_single_user_mode_username,
     Operations,
     SUCCESS_CODE,
-    TEST_NHS_NUMBER,
 )
 
 
+TEST_NHS_NUMBER = generate_random_nhs_number()
+
+
+def get_reply_dict_from_response(response: Response) -> Dict[str, str]:
+    """
+    For unit testing: convert the text in a :class:`Response` back to a
+    dictionary, so we can check it was correct.
+    """
+    txt = str(response)
+    d = {}  # type: Dict[str, str]
+    # Format is: "200 OK\r\n<other headers>\r\n\r\n<content>"
+    # There's a blank line between the heads and the body.
+    http_gap = "\r\n\r\n"
+    camcops_linesplit = "\n"
+    camcops_k_v_sep = ":"
+    try:
+        start_of_content = txt.index(http_gap) + len(http_gap)
+        txt = txt[start_of_content:]
+        for line in txt.split(camcops_linesplit):
+            if not line:
+                continue
+            colon_pos = line.index(camcops_k_v_sep)
+            key = line[:colon_pos]
+            value = line[colon_pos + len(camcops_k_v_sep):]
+            key = key.strip()
+            value = value.strip()
+            d[key] = value
+        return d
+    except ValueError:
+        return {}
+
+
 class ClientApiTests(DemoDatabaseTestCase):
     """
     Unit tests.
@@ -178,11 +215,7 @@ class ClientApiTests(DemoDatabaseTestCase):
                 self.fail(f"Operations.{x} fails validate_alphanum_underscore")
 
 
-class PatientRegistrationTests(DemoDatabaseTestCase):
-    def create_tasks(self) -> None:
-        # Speed things up a bit
-        pass
-
+class PatientRegistrationTests(BasicDatabaseTestCase):
     def test_returns_patient_info(self) -> None:
         import datetime
         patient = self.create_patient(
@@ -470,11 +503,7 @@ class PatientRegistrationTests(DemoDatabaseTestCase):
         self.assertEqual(ip_use_info[TabletParam.IP_USE_RESEARCH], 0)
 
 
-class GetTaskSchedulesTests(DemoDatabaseTestCase):
-    def create_tasks(self) -> None:
-        # Speed things up a bit
-        pass
-
+class GetTaskSchedulesTests(BasicDatabaseTestCase):
     def test_returns_task_schedules(self) -> None:
         from pendulum import DateTime as Pendulum, Duration, local, parse
 
