@@ -5,7 +5,8 @@ camcops_server/tasks/cape42.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/cape42.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -123,6 +124,7 @@ class Cape42(TaskHasPatientMixin, Task,
     __tablename__ = "cape42"
     shortname = "CAPE-42"
     provides_trackers = True
+    info_filename_stem = "cape"
 
     @staticmethod
     def longname(req: "CamcopsRequest") -> str:
