@@ -5,7 +5,8 @@ camcops_server/tasks/cardinal_expectationdetection.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/cardinal_expectationdetection.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -984,8 +985,8 @@ class CardinalExpectationDetection(TaskHasPatientMixin, Task):
                     <th>d'</th>
                 </tr>
         """
-        for half in [0, 1]:
-            for prob in [0, 1]:
+        for half in (0, 1):
+            for prob in (0, 1):
                 blocks = list(range(half * self.num_blocks // 2,
                                     self.num_blocks // (2 - half)))
                 groups = groups_lowprob if prob == 0 else groups_highprob
@@ -1039,7 +1040,7 @@ class CardinalExpectationDetection(TaskHasPatientMixin, Task):
                 </tr>
         """
         for b in range(self.num_blocks):
-            for prob in [0, 1]:
+            for prob in (0, 1):
                 groups = groups_lowprob if prob == 0 else groups_highprob
                 (p_detected_given_present,
                  p_detected_given_absent,
