@@ -5,7 +5,8 @@ camcops_server/tasks/cpft_lps.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/cpft_lps.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -96,6 +97,7 @@ class CPFTLPSReferral(TaskHasPatientMixin, Task):
     """
     __tablename__ = "cpft_lps_referral"
     shortname = "CPFT_LPS_Referral"
+    info_filename_stem = "clinical"
 
     referral_date_time = Column("referral_date_time",
                                 PendulumDateTimeAsIsoTextColType)
@@ -357,6 +359,7 @@ class CPFTLPSResetResponseClock(TaskHasPatientMixin, TaskHasClinicianMixin,
     """
     __tablename__ = "cpft_lps_resetresponseclock"
     shortname = "CPFT_LPS_ResetResponseClock"
+    info_filename_stem = "clinical"
 
     reset_start_time_to = Column(
         "reset_start_time_to", PendulumDateTimeAsIsoTextColType
@@ -411,6 +414,7 @@ class CPFTLPSDischarge(TaskHasPatientMixin, TaskHasClinicianMixin, Task):
     """
     __tablename__ = "cpft_lps_discharge"
     shortname = "CPFT_LPS_Discharge"
+    info_filename_stem = "clinical"
 
     discharge_date = Column("discharge_date", Date)
     discharge_reason_code = CamcopsColumn(
