@@ -5,7 +5,8 @@ camcops_server/tasks/demqol.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/demqol.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -302,6 +303,7 @@ class DemqolProxy(TaskHasPatientMixin, TaskHasRespondentMixin,
     __tablename__ = "demqolproxy"
     shortname = "DEMQOL-Proxy"
     extrastring_taskname = "demqol"
+    info_filename_stem = "demqol"
 
     q32 = CamcopsColumn(
         "q32", Integer,
