@@ -5,7 +5,8 @@ camcops_server/tasks/honos.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/honos.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -185,6 +186,7 @@ class Honos(HonosBase,
     """
     __tablename__ = "honos"
     shortname = "HoNOS"
+    info_filename_stem = "honos"
 
     q8problemtype = CamcopsColumn(
         "q8problemtype", CharColType,
@@ -359,6 +361,7 @@ class Honos65(HonosBase,
     """
     __tablename__ = "honos65"
     shortname = "HoNOS 65+"
+    info_filename_stem = "honos"
 
     q8problemtype = CamcopsColumn(
         "q8problemtype", CharColType,
@@ -523,6 +526,7 @@ class Honosca(HonosBase,
     """
     __tablename__ = "honosca"
     shortname = "HoNOSCA"
+    info_filename_stem = "honos"
 
     NQUESTIONS = 15
     QFIELDS = strseq("q", 1, NQUESTIONS)
