@@ -5,7 +5,8 @@ camcops_server/tasks/maas.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/maas.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -264,12 +265,12 @@ class Maas(TaskHasPatientMixin, Task,
           <div class="{CssClass.FOOTNOTES}">
             Condon, J. (2015). Maternal Antenatal Attachment Scale
             [Measurement instrument]. Retrieved from <a
-            href="http://hdl.handle.net/2328/35292">http://hdl.handle.net/2328/35292</a>.
+            href="https://hdl.handle.net/2328/35292">https://hdl.handle.net/2328/35292</a>.
 
             Copyright © John T Condon 2015. This is an Open Access article
             distributed under the terms of the Creative Commons Attribution
             License 3.0 AU (<a
-            href="http://creativecommons.org/licenses/by/3.0">http://creativecommons.org/licenses/by/3.0</a>),
+            href="https://creativecommons.org/licenses/by/3.0">https://creativecommons.org/licenses/by/3.0</a>),
             which permits unrestricted use, distribution, and reproduction in
             any medium, provided the original work is properly cited.
           </div>
