@@ -5,7 +5,8 @@ camcops_server/tasks/rand36.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/rand36.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -489,7 +490,7 @@ class Rand36(TaskHasPatientMixin, Task,
         for q in range(23, 31 + 1):
             h += self.answer_row_html(req, q)
         h += self.section_row_html("<br>")
-        for q in [32]:
+        for q in (32, ):
             h += self.answer_row_html(req, q)
         h += self.section_row_html(self.wxstring(req, "q33to36stem"))
         for q in range(33, 36 + 1):
@@ -499,7 +500,8 @@ class Rand36(TaskHasPatientMixin, Task,
             <div class="{CssClass.COPYRIGHT}">
                 The RAND 36-Item Short Form Health Survey was developed at RAND
                 as part of the Medical Outcomes Study. See
-            http://www.rand.org/health/surveys_tools/mos/mos_core_36item.html
+                <a href="https://www.rand.org/health/surveys_tools/mos/mos_core_36item.html">
+                https://www.rand.org/health/surveys_tools/mos/mos_core_36item.html</a>
             </div>
             <div class="{CssClass.FOOTNOTES}">
                 All questions are first transformed to a score in the range
@@ -514,5 +516,5 @@ class Rand36(TaskHasPatientMixin, Task,
                 [8] Q21, 22.
                 [9] Q1, 33–36.
             </div>
-        """
+        """  # noqa
         return h
