@@ -5,7 +5,8 @@ camcops_server/tasks/ybocs.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tasks/ybocs.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -70,7 +71,7 @@ class YbocsMetaclass(DeclarativeMeta):
                  bases: Tuple[Type, ...],
                  classdict: Dict[str, Any]) -> None:
         cls.TARGET_COLUMNS = []  # type: List[Column]
-        for target in ["obsession", "compulsion", "avoidance"]:
+        for target in ("obsession", "compulsion", "avoidance"):
             for n in range(1, cls.NTARGETS + 1):
                 fname = f"target_{target}_{n}"
                 col = Column(
@@ -328,6 +329,7 @@ class YbocsSc(TaskHasClinicianMixin, TaskHasPatientMixin, Task,
     __tablename__ = "ybocssc"
     shortname = "Y-BOCS-SC"
     extrastring_taskname = "ybocs"  # shares with Y-BOCS
+    info_filename_stem = extrastring_taskname
 
     SC_PREFIX = "sc_"
     SUFFIX_CURRENT = "_current"
