@@ -5,7 +5,8 @@ camcops_server/tools/fetch_snomed_codes.py
 
 ===============================================================================
 
-    Copyright (C) 2012-2020 Rudolf Cardinal (rudolf@pobox.com).
+    Copyright (C) 2012, University of Cambridge, Department of Psychiatry.
+    Created by Rudolf Cardinal (rnc1001@cam.ac.uk).
 
     This file is part of CamCOPS.
 
@@ -20,7 +21,7 @@ camcops_server/tools/fetch_snomed_codes.py
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with CamCOPS. If not, see <http://www.gnu.org/licenses/>.
+    along with CamCOPS. If not, see <https://www.gnu.org/licenses/>.
 
 ===============================================================================
 
@@ -29,7 +30,7 @@ identifiers from a SNOMED server (e.g. of their national provider).
 
 References:
 
-- http://ihtsdo.github.io/sct-snapshot-rest-api/api.html
+- https://ihtsdo.github.io/sct-snapshot-rest-api/api.html
 - https://termbrowser.nhs.uk/
 
 """
@@ -44,6 +45,7 @@ from typing import Dict, List, Optional, Set, Tuple, Union
 
 from cardinal_pythonlib.argparse_func import ShowAllSubparserHelpAction
 from cardinal_pythonlib.json.typing_helpers import JsonValueType
+from cardinal_pythonlib.httpconst import HttpStatus
 from cardinal_pythonlib.logs import main_only_quicksetup_rootlogger
 from cardinal_pythonlib.rate_limiting import rate_limited
 from cardinal_pythonlib.snomed import SnomedConcept
@@ -59,13 +61,11 @@ log = logging.getLogger(__name__)
 
 DEFAULT_URL = "https://termbrowser.nhs.uk/sct-browser-api/snomed"
 DEFAULT_EDITION = "uk-edition"
-DEFAULT_RELEASE = "v20191001"
+DEFAULT_RELEASE = "v20210929"
 DEFAULT_LANGUAGE = "english"
 DEFAULT_RATE_LIMIT_HZ = 1  # be nice
 DEFAULT_OUTPUT_XML_FILENAME = "camcops_tasks_snomed.xml"
 
-HTTP_200_OK = 200
-
 DISCLAIMER_1 = """
 SNOMED Clinical Terms® (SNOMED CT®) is owned by the International Health
 Terminology Standards Development Organization (IHTSDO), trading as SNOMED
@@ -74,7 +74,7 @@ International. All rights reserved.
 Some people/organizations in some countries are authorized to use SNOMED CT
 identifiers for data creation systems. Others are not. See e.g.
 
-    http://snomed.org/licensing
+    https://snomed.org/licensing
     https://www.snomed.org/snomed-ct/get-snomed
     https://termbrowser.nhs.uk
 
@@ -219,6 +219,8 @@ SWEMWBS2 = "Short Warwick-Edinburgh Mental Well-Being Scale"
 WSAS = "Improving Access to Psychological Therapies programme Work and Social Adjustment Scale"  # noqa
 
 CCMAP = OrderedDict([
+    # camcops_name, snomed_term_details (= term_name, semantic_type, term_suffix)  # noqa
+    (SL.OBSERVABLE_ENTITY, ("Observable entity", ST.OBS)),
     (SL.MASS, ("Mass, a measure of quantity of matter", ST.QV, " (property)")),
     (SL.LENGTH, ("Length", ST.QV, " property")),
     (SL.UNIT_OF_MEASURE, ("Unit of measure", ST.QV)),
@@ -290,7 +292,10 @@ CCMAP = OrderedDict([
     (SL.PHOTOGRAPH_PROCEDURE, ("Medical photography", ST.PROC)),
     (SL.PHOTOGRAPH_PHYSICAL_OBJECT, ("Photograph", ST.OBJ)),
 
-    (SL.PSYCHIATRIC_ASSESSMENT_PROCEDURE, (f"Psychiatric diagnostic interview, examination, history, mental status and disposition", ST.PROC)),  # noqa
+    # Deprecated between v20191001 and v20210929:
+    # (SL.PSYCHIATRIC_ASSESSMENT_PROCEDURE, (f"Psychiatric diagnostic interview, examination, history, mental status and disposition", ST.PROC)),  # noqa
+    # Its replacement in v20210929:
+    (SL.DIAGNOSTIC_PSYCHIATRIC_INTERVIEW_PROCEDURE, (f"Diagnostic psychiatric interview", ST.PROC)),  # noqa
 
     (SL.PSYCLERK_REASON_FOR_REFERRAL, ("Reason for referral", ST.REC)),
     (SL.PSYCLERK_PRESENTING_ISSUE, ("Presenting complaints or issues", ST.REC)),  # NOQA
@@ -313,7 +318,8 @@ CCMAP = OrderedDict([
     (SL.PSYCLERK_MSE_BEHAVIOUR, ("Behavior observable", ST.OBS)),
     (SL.PSYCLERK_MSE_SPEECH, ("Speech observable", ST.OBS)),
     (SL.PSYCLERK_MSE_MOOD, ("Mood, function", ST.OBS)),
-    (SL.PSYCLERK_MSE_AFFECT, ("Affect, function", ST.OBS)),
+    (SL.PSYCLERK_MSE_AFFECT, ("Affect function", ST.OBS)),
+    # ... was "Affect, function" in v20191001; changed for v20210929.
     (SL.PSYCLERK_MSE_THOUGHT, ("Thinking, function", ST.OBS)),
     (SL.PSYCLERK_MSE_PERCEPTION, ("Perception, function", ST.OBS)),
     (SL.PSYCLERK_MSE_COGNITION, ("Cognitive functions", ST.OBS)),
@@ -546,7 +552,7 @@ class SnomedApiInfo(object):
         """
         log.debug(f"Fetching from {url}")
         response = requests.get(url, params)
-        if response.status_code != HTTP_200_OK:
+        if response.status_code != HttpStatus.OK:
             log.warning(f"Response was: {response}")
             return None
         json_object = response.json()
@@ -628,12 +634,14 @@ class SnomedApiInfo(object):
         log.debug(f"Query params: {params}")
         json_object = self.get_result(url, params)
         if not isinstance(json_object, dict):
-            raise ConceptNotFound(f"No result for {term!r}.")
+            raise ConceptNotFound(f"No result for {term!r} "
+                                  f"(semantic_area={semantic_area!r}.")
         matches = json_object["matches"]
         if not isinstance(matches, list):
             raise ConceptNotFound(f"Bad 'matches' object for {term!r}.")
         if len(matches) == 0:
-            raise ConceptNotFound(f"No matches for {term!r}.")
+            raise ConceptNotFound(f"No matches for {term!r} "
+                                  f"(semantic_area={semantic_area!r}.")
         log.debug(f"Found {len(matches)} hits for {term!r}")
         for match in matches:
             assert isinstance(match, dict)
@@ -669,29 +677,29 @@ def test_api(api: SnomedApiInfo,
     concept_ids = concept_ids or []  # type: List[int]
     terms = terms or []  # type: List[str]
 
-    log.warning("Running tests...")
+    log.info("Running tests...")
 
     if not concept_ids and not terms:
-        log.warning("Using default tests")
+        log.info("Using default tests")
 
-        log.warning("'Pneumonia (disorder)' by ID")
+        log.info("'Pneumonia (disorder)' by ID")
         api.get_concept_by_id("233604007")
 
-        log.warning("'Lupus pneumonia (disorder)' by name")
+        log.info("'Lupus pneumonia (disorder)' by name")
         api.get_concept_by_term("Lupus pneumonia", SemanticTypes.DIS)
 
-        log.warning("'Pneumonia (disorder)' by name")
+        log.info("'Pneumonia (disorder)' by name")
         api.get_concept_by_term("Pneumonia", SemanticTypes.DIS)
 
     for cid in concept_ids:
-        log.warning(str(cid))
+        log.info(str(cid))
         api.get_concept_by_id(cid)
 
     for term in terms:
         if semantic_area:
-            log.warning(f"{term} ({semantic_area})")
+            log.info(f"{term} ({semantic_area})")
         else:
-            log.warning(f"{term}")
+            log.info(f"{term}")
         api.get_concept_by_term(term, semantic_area)
 
 
@@ -755,20 +763,24 @@ def fetch_camcops_snomed_codes(api: SnomedApiInfo,
     seen = set()  # type: Set[str]
     with open(filename, "wt") as f:
         print(XML_HEADER, file=f)
-        for camcops_name, name_type in CCMAP.items():
-            sname = name_type[0]
-            stype = name_type[1]
-            suffix = name_type[2] if len(name_type) >= 3 else ""
+        for camcops_name, snomed_term_details in CCMAP.items():
+            term_name = snomed_term_details[0]
+            semantic_type = snomed_term_details[1]
+            term_suffix = (
+                snomed_term_details[2] if len(snomed_term_details) >= 3 else ""
+            )
             assert camcops_name in possible_names, f"Bad name: {camcops_name}"
-            assert stype in possible_types, f"Bad type: {stype}"
+            assert semantic_type in possible_types, f"Bad type: {semantic_type}"
             try:
-                concept = api.get_concept_by_term(sname, stype, suffix=suffix)
+                concept = api.get_concept_by_term(
+                    term_name, semantic_type, suffix=term_suffix)
                 xml = get_xml(camcops_name, concept)
                 print(f"    {xml}", file=f)
                 seen.add(camcops_name)
             except ConceptNotFound:
                 if continue_on_error:
-                    log.error(f"Could not find: {sname!r}, {stype!r}")
+                    log.error(f"Could not find: "
+                              f"{term_name!r}, {semantic_type!r}")
                 else:
                     raise
         print(XML_FOOTER, file=f)
